/*----------------------Web Analytics-----------------------------------------*/
/* WAFUNNEL                                                                   */
/* Create the Interactive Funnel Report                                       */
/*                                                                            */
/*    two calls are used: the 1st (&INFO eq YES) creates a template data set  */
/*    that serves are the model for the report data set, which is created     */
/*    by the 2nd call.                                                        */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Supported By: Frank Roediger, FRROED; Kevin Scott, SASKXS                  */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* History:                                                                   */
/* Date          Description                    Username         Change Code  */
/* 20040331      Program Created                 frroed              NA       */
/* 20040608      Change value of ONETHENTWO...   frroed              NA       */
/* 20050330      S0288450                        frroed              NA       */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Dependencies:                                                              */
/*    The following macros need to be available:                              */
/*      -- none --                                                            */
/*                                                                            */
/*    The following macro variables need to be defined and assigned values    */
/*       by the User Interface/Middleware                                     */
/*    &INFO:       a switch variable that indicates whether to create a       */
/*                 place-holding (1 obs with missing values) or a populated   */
/*                 (summarization of actual data) version of &OUTPUT_DATASET  */
/*    &LEVELONE:   the reference to the REQUESTED_FILE that is the first      */
/*                 entry in the funnel.  the $IDXTORF format associates the   */
/*                 reference number with the actual REQUESTED_FILE.  there    */
/*                 are seven of these references, &LEVELONE - &LEVELSEVEN     */
/*    &ONETHENTWO: a switch variable that indicates whether the &LEVELONE and */
/*                 &LEVELTWO pages need to have occurred consecutively in     */
/*                 order to be tallied in the analysis. there are six of      */
/*                 these switches, &ONETHENTWO - &SIXTHENSEVEN.               */
/*    &DATES:      a comma-delimited list of the YYYYMMDD values that are the */
/*                 dates for the analysis.                                    */
/*    &LIBREFS:    a comma-delimited list of the SAS libnames that need to    */
/*                 be defined by the process.                                 */
/*    &LIBPATHS:   a comma-delimited list of the locations for the            */
/*                 corresponding libnames in the LIBREFS list.                */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Parameters:                                                                */
/*      -- none --                                                            */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Input:                                                                     */
/*      -- none --                                                            */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Output:                                                                    */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Copyright (c) 2004 SAS Institute Inc. All Rights Reserved                  */
/*----------------------------------------------------------------------------*/
%macro wafunnel;

%* parse the LIBREFS and LIBPATHS strings and assign necessary librefs         ;
data _null_;
  length tmprefs  $ 1024
         tmppaths $ 4096;

   tmprefs="&librefs";
   tmppaths="&libpaths";

   %* make sure that both lists have the same number of elements               ;
   num_ref_commas=countc(tmprefs,',');
   num_path_commas=countc(tmppaths,',');

   if num_ref_commas eq num_path_commas then do;
      do i=1 to (1+num_ref_commas);
         libname=scan(tmprefs,i,',');
         libpath=strip(scan(tmppaths,i,','));
         rc = filename("fileref",libpath);
         did = dopen("fileref");
         if did then do;
            rc=dclose(did);
            rc = libname(strip(libname), libpath);
            call symput("_wab_lib_error_","0");
         end;
         else do;
            call symput("_wab_lib_error_","2");
            call symput("_wab_lib_msg_"
                       ,"The physical location "||strip(libpath)
                      ||" either does not exist or cannot be "
                      ||"opened. The library "||strip(libname)
                      ||" must be respecified with a valid location.");
         end;
      end;
   end;
   else do;
      call symput("_wab_lib_error_","3");
      call symput("_wab_lib_msg_","You have different numbers of "
                                  ||"library references than physical paths."
                                  ||"Please respecify your libraries.");
   end;

run;
%* S0257525 -- WAFUNNEL HAS MORE EXTENSIVE ERROR CHECKING THAN IS NECESSARY    ;
%*    this block has been commented out to address defect S0257525             ;
%*%if &_wab_lib_error_ ne 0 %then %do;
%*   %put %unquote(&wab_error) &_wab_lib_msg_;
%*   %goto done;
%*%end;

%* create a template data set for the STP to use as a model                    ;
data _null_;
   if strip(lowcase(compress("&info")))="true" then
      call symput("info","TRUE");
   else
      call symput("info","");
run;
%if %length(&info) gt 1 %then %do;
   data _null_;
      %* create a unique id for the dsn that will contain the funnel template  ;
      call symput("_funnel_data_uid_"
                 ,compress(
                           translate(   strip(put(datetime(),15.))
                                     || "_"
                                     || strip(put(ranuni(-1),13.11))
                                    ,' '
                                    ,'.'
                                    )
                          )
                 );
   run;

   data stp.template_&_funnel_data_uid_;
      length _name_      $10
             desc        $50
             sessions
             sessions2
             perc_level1
             perc_prev
             dropoff      8;
      format sessions
             sessions2   nlnum8.
             perc_prev
             dropoff
             perc_level1 nlnum5.2;
      label _name_ = "Process"
            event = "Rule"
            desc = "Event"
            sessions = "Total Visits"
            sessions2 = "Path Visits"
            perc_level1 = "% of First Level"
            perc_prev = "% of Previous Level"
            dropoff = "% of Drop Off"
           ;
   run;

   %let outdsn=stp.template_&_funnel_data_uid_;

   %goto done;
%end;


%* create the $IDXTORF format that identifies REQUESTED_FILEs by their index   ;
proc format cntlin=summary.page_frequencies;
run;


%* assign values to macro variables that identify the funnel components        ;
data _null_;

   length temp_str $ 3200;

   %* create a unique id for the dsn that will contain the funnel analysis     ;
   call symput("_funnel_data_uid_"
              ,compress(
                        translate(   strip(put(datetime(),15.))
                                  || "_"
                                  || strip(put(ranuni(-1),13.11))
                                 ,' '
                                 ,'.'
                                 )
                       )
              );

   %* create FUNNEL_LIST, which will be used in an IN list to subset only the  ;
   %*    pathing obs that pertain to the designated funnel elements            ;
   if "&levelone" gt ' ' then
      temp_str="'"
            || strip("&levelone")
            || "'";
   if "&leveltwo" gt ' ' then
      temp_str=strip(temp_str)
            || ",'"
            || strip("&leveltwo")
            || "'";
   if "&levelthree" gt ' ' then
      temp_str=strip(temp_str)
            || ",'"
            || strip("&levelthree")
            || "'";
   if "&levelfour" gt ' ' then
      temp_str=strip(temp_str)
            || ",'"
            || strip("&levelfour")
            || "'";
   if "&levelfive" gt ' ' then
      temp_str=strip(temp_str)
            || ",'"
            || strip("&levelfive")
            || "'";
   if "&levelsix" gt ' ' then
      temp_str=strip(temp_str)
            || ",'"
            || strip("&levelsix")
            || "'";
   if "&levelseven" gt ' ' then
      temp_str=strip(temp_str)
            || ",'"
            || strip("&levelseven")
            || "'";
   call symput('funnel_list',strip(temp_str));

   %* use the $IDXTORF format to identify the referrer page associated with    ;
   %*    each code passed by the UI                                            ;
   %* NOTE: any embedded single quotes are TRANWRDed into paired single quotes ;
   temp_str=tranwrd(put(strip("&levelOne"),$idxtorf.),"'","''");
   call symput("level01",strip(temp_str));

   temp_str=tranwrd(put(strip("&levelTwo"),$idxtorf.),"'","''");
   call symput("level02",strip(temp_str));

   temp_str=tranwrd(put(strip("&levelThree"),$idxtorf.),"'","''");
   call symput("level03",strip(temp_str));

   temp_str=tranwrd(put(strip("&levelFour"),$idxtorf.),"'","''");
   call symput("level04",strip(temp_str));

   temp_str=tranwrd(put(strip("&levelFive"),$idxtorf.),"'","''");
   call symput("level05",strip(temp_str));

   temp_str=tranwrd(put(strip("&levelSix"),$idxtorf.),"'","''");
   call symput("level06",strip(temp_str));

   temp_str=tranwrd(put(strip("&levelSeven"),$idxtorf.),"'","''");
   call symput("level07",strip(temp_str));

   call symput("twocontig","");
   call symput("threecontig","");
   call symput("fourcontig","");
   call symput("fivecontig","");
   call symput("sixcontig","");
   call symput("sevencontig","");

   if left(trim(lowcase(compress("&oneThenTwo"))))="on" then
      call symput("twocontig","yes");
   if left(trim(lowcase(compress("&twoThenThree"))))="on" then
      call symput("threecontig","yes");
   if left(trim(lowcase(compress("&threeThenFour"))))="on" then
      call symput("fourcontig","yes");
   if left(trim(lowcase(compress("&fourThenFive"))))="on" then
      call symput("fivecontig","yes");
   if left(trim(lowcase(compress("&fiveThenSix"))))="on" then
      call symput("sixcontig","yes");
   if left(trim(lowcase(compress("&sixThenSeven"))))="on" then
      call symput("sevencontig","yes");

/*  SAMPLE subsets are not part of Web Analytics */
*  if exist("summary.pathing_sample_h") then do;
*     path_dsn="summary.pathing_sample_h";
*     sample=1;
*  end;
*  else do;
*     path_dsn="summary.pathing_h";
*     sample=0;
*  end;

   call symput('path_dsn','summary.pathing');
   call symput('samp','0');

run;


*% create metadata using the funnel parameters passed by the UI                ;
data d&_funnel_data_uid_;
   length events
          desc   $ 128;
   format events
          desc   $128.;

   %if %length(%bquote(&level01)) gt 0 %then %do;
      desc="&level01";
      contiguous=0;
      events="requested_file='&level01'";
      output;
   %end;

   %if %length(%bquote(&level02)) gt 0 %then %do;
      events="requested_file='&level02'";
      desc="&level02";
      contiguous=0;
      %if %length(%cmpres(&TwoContig)) gt 2 %then %do;
         contiguous=1;
      %end;
      output;
   %end;

   %if %length(%bquote(&level03)) gt 0 %then %do;
      events="requested_file='&level03'";
      desc="&level03";
      contiguous=0;
      %if %length(%cmpres(&ThreeContig)) gt 2 %then %do;
         contiguous=1;
      %end;
      output;
   %end;

   %if %length(%bquote(&level04)) gt 0 %then %do;
      events="requested_file='&level04'";
      desc="&level04";
      contiguous=0;
      %if %length(%cmpres(&FourContig)) gt 2 %then %do;
         contiguous=1;
      %end;
      output;
   %end;

   %if %length(%bquote(&level05)) gt 0 %then %do;
      events="requested_file='&level05'";
      desc="&level05";
      contiguous=0;
      %if %length(%cmpres(&FiveContig)) gt 2 %then %do;
         contiguous=1;
      %end;
      output;
   %end;

   %if %length(%bquote(&level06)) gt 0 %then %do;
      events="requested_file='&level06'";
      desc="&level06";
      contiguous=0;
      %if %length(%cmpres(&SixContig)) gt 2 %then %do;
         contiguous=1;
      %end;
      output;
   %end;

   %if %length(%bquote(&level07)) gt 0 %then %do;
      events="requested_file='&level07'";
      desc="&level07";
      contiguous=0;
      %if %length(%cmpres(&SevenContig)) gt 2 %then %do;
         contiguous=1;
      %end;
      output;
   %end;

run;


%* If any ADJACENT checkbox has been checked in the UI, make sure that the     ;
%*    &CONTIG macro variable is set to TRUE.                                   ;
%* NOTE: the CONTIGUOUS field is set to 1 if ADJACENT has been checked and is  ;
%*       set to 0 if it has not.  any non-zero sum indicates that one of the   ;
%*       ADJACENT checkboxes has been checked.                                 ;
proc summary data=D&_funnel_data_uid_ chartype noprint;
   var contiguous;
   output out=contig_&_funnel_data_uid_
          sum=contig_ttl;
run;
data _null_;
   set contig_&_funnel_data_uid_;
   if contig_ttl then
      contig='TRUE';
   else
      contig='F';
   call symput("contig",trim(left(contig)));
run;


%* create macro variables with the values from the funnel metadata             ;
data _null_;
   set d&_funnel_data_uid_ end=eof;

   length cond_list $50;
   retain cond_list;

   if _n_ eq 1 then do;
      cond_list="'"
             || strip(put(contiguous,6.))
             || "'";
   end;
   else do;
      cond_list=strip(cond_list)
             || ",'"
             || strip(put(contiguous,6.))
             || "'";
   end;

   call symput(compress('event'||
                        put(_n_,3.))
              ,trim(left(events)));
   call symput(compress('desc'||
                        put(_n_,3.))
              ,trim(left(desc)));
   call symput(compress('condition'||
                        put(_n_,3.))
              ,trim(left(put(contiguous,6.))));

   if eof then do;
      call symput('total_events',trim(left(put(_n_,3.))));
      call symput('cond_list',strip(cond_list));
      %* determine if all ADJACENT checkboxes have the same status             ;
      %*    NOTE: the 1st element occupies the 1st four bytes in the comma-    ;
      %*          delimited COND_LIST.  it is always zero, so it is not        ;
      %*          considered when checking the ADJACENT checkboxes status.     ;
      temp_list=substr(cond_list,5);
      if index(temp_list,'0') and
         index(temp_list,'1') then
         call symput('same_adj','0');
      else
         call symput('same_adj','1');
   end;

run;


%* convert &DATES (YYYYMMDD) to &SASDATES (SAS date values)                    ;
options noquotelenmax;      /* avoid warning message about unbalanced quotes  */
data _null_;
   length sas_dates $ 3200;
   date_str="&dates";

   date_cnt=1+countc(date_str,',');

   sas_dates=put(input(scan(date_str,1,','),yymmdd10.),5.);

   do i=2 to date_cnt;
      curr_date=scan(date_str,i,',');
      sas_dates=strip(sas_dates)
             || ','
             || put(input(scan(date_str,i,','),yymmdd10.),5.);
   end;

   call symput('sasdates',strip(sas_dates));

run;


%* --------------------------------------------------------------------------- ;
%*    this block handles the cases where some (not all) of the Adjacent        ;
%*       checkboxes have been checked.  it is a separate block because         ;
%*       modifying the existing code to accommodate these conditions was       ;
%*       not feasible in the available time-frame.                             ;
%* --------------------------------------------------------------------------- ;
%if &same_adj eq 0 %then %do;
%* create a subset extract from &PATH_DSN with just the funnel elements for    ;
%*    the designated dates                                                     ;
data extr_f&_funnel_data_uid_;
   set &path_dsn
      (where=(date in (&sasdates)));
run;


%* create the $FUN_REF format to identify designated funnel elements           ;
data cntlin_&_funnel_data_uid_;
   length fmtname $ 8
          start   $ 12
          label   $ 1;

   fmtname='$fun_ref';

   label='1';
   start="&LevelOne";
   if start gt ' ' then
      output;

   label='2';
   start="&LevelTwo";
   if start gt ' ' then
      output;

   label='3';
   start="&LevelThree";
   if start gt ' ' then
      output;

   label='4';
   start="&LevelFour";
   if start gt ' ' then
      output;

   label='5';
   start="&LevelFive";
   if start gt ' ' then
      output;

   label='6';
   start="&LevelSix";
   if start gt ' ' then
      output;

   label='7';
   start="&LevelSeven";
   if start gt ' ' then
      output;

   hlo='O';
   label='0';
   output;

run;
proc format cntlin=cntlin_&_funnel_data_uid_;
run;


data in_f&_funnel_data_uid_;
   set extr_f&_funnel_data_uid_
       (keep=session_id
            date
            reference
            sequence
       );
   by session_id;

   length ckbox1  -ckbox&total_events   $ 1;

   array ckbox    {1:&total_events} $ ckbox1   -ckbox&total_events;
   array level    {1:&total_events}   level1   -level&total_events;
   array funnel   {1:&total_events}   funnel1  -funnel&total_events;
   array hold_fun {1:&total_events}   hold_fun1-hold_fun&total_events;

   retain level1   -level&total_events
          funnel1  -funnel&total_events
          hold_fun1-hold_fun&total_events
          max_hold
          curr_hold
          ckbox1-ckbox&total_events
         ;

   if _n_ eq 1 then do;
      do i=1 to &total_events;
         %* break out adj checkboxed from  CHECKBOXES                          ;
         ckbox{i}=strip(tranwrd(scan("&cond_list",i,','),"'"," "));
      end;
   end;

   %* initialize the active funnel storage area                                ;
   if first.session_id then do;
      do i=1 to &total_events;
         level{i}   =0;
         funnel{i}  =0;
         hold_fun{i}=0;
      end;
      max_hold=0;
      curr_hold=1;
   end;

   element_sw=put(reference,$fun_ref.);
   %* ------------------------------------------------------------------------ ;
   %* current element is a funnel event                                        ;
   %* ------------------------------------------------------------------------ ;
   if element_sw gt '0' then do;
      curr_pos=input(element_sw,1.);
      level{curr_pos}=1;
      if curr_hold eq curr_pos then do;
         %* register when an in-funnel entry occurs in sequence                ;
         funnel{curr_hold}= 1;
         curr_hold+1;
         %* dump when all funnel elements have been encountered                ;
         if curr_hold gt &total_events then do;
            curr_hold+(-1);
            link DUMP;
         end;
      end;
      else do;
         %* dump when out-of-order funnel entry occurs                         ;
         link DUMP;
         %* register when 1st funnel entry has recurred out-of-order           ;
         if curr_pos eq 1 then do;
            funnel{curr_hold}= 1;
            curr_hold+1;
         end;
      end;
   end;
   %* ------------------------------------------------------------------------ ;
   %* current element is NOT a funnel event                                    ;
   %* ------------------------------------------------------------------------ ;
   else do;
      if ckbox{curr_hold} eq '0' then do;
         %* ignore non-funnel entry when current funnel element does not need  ;
         %*    to be contiguous                                                ;
      end;
      else do;
         %* dump when non-funnel entry occurs where contiguity is required     ;
         link DUMP;
      end;
   end;

   if last.session_id then do;
      %* move the active funnel storage area to HOLD, if necessary             ;
      link DUMP;
      %* write out the sessions most complete encounter with the funnel        ;
      do i=1 to &total_events;
         %* LEVELx is not in HOLD -- it indicates any encounter                ;
         funnel{i}  =hold_fun{i};
      end;
      output;
   end;

   drop i
        j
        ckbox1-ckbox&total_events
        hold_fun1-hold_fun&total_events
        max_hold
        curr_hold
        element_sw
        curr_pos
        reference
        sequence
       ;

   return;

DUMP:;
   %* Dump the active funnel storage area into HOLD                            ;
   if curr_hold gt max_hold then do;
      do j=1 to curr_hold;
         hold_fun{j}=funnel{j};
      end;
      max_hold=curr_hold;
   end;
   %* Re-initialize the active funnel storage area                             ;
   do i=1 to &total_events;
      %* LEVELx is not re-initialized -- it indicates any encounter            ;
      funnel{i}  =0;
   end;
   curr_hold=1;

   return;

run;
%end;
%* --------------------------------------------------------------------------- ;
%*    this block handles the cases where all of the Adjacent checkboxes have   ;
%*       the same status (either checked or non-checked).  it is a separate    ;
%*       block because this code was already in place and worked without       ;
%*       modification for these two scenarios but could not be readily         ;
%*       changed to handle the additional scenario of mixed-status Adjacent    ;
%*       checkboxes                                                            ;
%* --------------------------------------------------------------------------- ;
%else %do;
%* create a subset extract from &PATH_DSN with just the funnel elements for    ;
%*    the designated dates                                                     ;
data extr_f&_funnel_data_uid_;
   set &path_dsn
      (where=(date      in (&sasdates) and
              reference in (&funnel_list)
      ));
run;


%* --------------------------------------------------------------------------- ;
%* process the subset extract to create variables for the funnel analysis:     ;
%*       LEVELx    - a 0/1 indicator of whether the corresponding REQUESTED_   ;
%*                   FILE is in the SESSION_ID                                 ;
%*       FUNNELx   - a 0/1 indicator of whether the corresponding LEVELx and   ;
%*                   all previous levels have been detected                    ;
%*       LEVELSS_x - contains the latest SEQUENCE of the SESSION_ID where the  ;
%*                   corresponding LEVELx has been encountered                 ;
%*    the EVENTx macro variables contain comparisons in the form:              ;
%*       REQUESTED_FILE="&LEVEL0x"                                             ;
%*    the CONDITIONx macro variables contain indicators of whether the         ;
%*       corresponding REQUESTED_FILE needs to be contiguous with the funnels  ;
%*       previous REQUESTED_FILE                                               ;
%* --------------------------------------------------------------------------- ;
data in_f&_funnel_data_uid_;
   set extr_f&_funnel_data_uid_
       (keep=session_id
            date
            requested_file
            sequence
       );
   by session_id;

   array level   {1:&total_events} level1   -level&total_events;
   array funnel  {1:&total_events} funnel1  -funnel&total_events;
   array levelss {1:&total_events} levelss_1-levelss_&total_events;

   array hold_lvl {1:&total_events} hold_lvl1-hold_lvl&total_events;
   array hold_fun {1:&total_events} hold_fun1-hold_fun&total_events;
   array hold_lss {1:&total_events} hold_lss1-hold_lss&total_events;

   retain level1   -level&total_events
          funnel1  -funnel&total_events
          levelss_1-levelss_&total_events
          hold_lvl1-hold_lvl&total_events
          hold_fun1-hold_fun&total_events
          hold_lss1-hold_lss&total_events
          max_hold
          curr_hold
         ;

   %* initialize the active funnel storage area                                ;
   if first.session_id then do;
      do i=1 to &total_events;
         level{i}   =0;
         funnel{i}  =0;
         levelss{i} =.;
         %* initialize the HOLD area (for funnels with contiguous elements)    ;
         if upcase("&contig") eq 'TRUE' then do;
            hold_lvl{i}=0;
            hold_fun{i}=0;
            hold_lss{i}=.;
         end;
      end;
      max_hold=0;
      curr_hold=0;
   end;

   %* Is the current REQUESTED_FILE the 1st funnel event?                      ;
   %*    Does the 2nd funnel element need to be contiguous with the 1st?       ;
   %*       - Dump the active funnel storage area into HOLD.                   ;
   %*       - Re-initialize the active funnel storage area.                    ;
   %*    Set the 1st event total indicator to 1.                               ;
   %*    Set the 1st funnel indicator to 1.                                    ;
   %*    Set the 1st sequence comparison integer to SEQUENCE -- the request    ;
   %*       number in the session where the first event occurs.                ;
   if &event1 then do;
      if upcase("&contig") eq 'TRUE' then do;
         if &condition2 then do;
            %* Dump the active funnel storage area into HOLD                   ;
            if curr_hold gt max_hold then do;
               do j=1 to curr_hold;
                  hold_lvl{j}=1;
                  hold_fun{j}=1;
                  hold_lss{j}=levelss{j};
               end;
               max_hold=curr_hold;
            end;
            %* Re-initialize the active funnel storage area                    ;
            do i=1 to &total_events;
               %* LEVELx is not re-initialized -- it indicates any encounter   ;
               funnel{i}  =0;
               levelss{i} =.;
               curr_hold=0;
            end;
         end;
      end;
      level1 = 1;
      funnel1 = 1;
      levelss_1 = sequence;
      curr_hold+1;
   end;

   %* Is the current REQUESTED_FILE a subsequent (not 1st) funnel event?       ;
   %do curr_lvl = 2 %to &total_events;
      %let prev_lvl = %eval(&curr_lvl - 1);

      %* IF the contiguous indicator (&CONDITIONx) is NOT on:                  ;
      %*    If the current REQUESTED_FILE is a funnel event:                   ;
      %*       Set the current event total indicator to 1.                     ;
      %*       Set the current sequence comparison integer to SEQUENCE -- the  ;
      %*          request number in the session where the current event occurs ;
      %*       If the previous events FUNNELx indicator is on then:            ;
      %*          Set the current funnel indicator to 1.                       ;
      if &&condition&curr_lvl ne 1 then do;
         if &&event&curr_lvl then do;
            level&curr_lvl = 1;
            levelss_&curr_lvl=sequence;
            if funnel&prev_lvl = 1 then
               funnel&curr_lvl = 1;
            curr_hold+1;
         end;
      end;

      %* ELSE (the contiguous indicator--&CONDITIONx--is on):                  ;
      %*    If the current REQUESTED_FILE is a funnel event:                   ;
      %*       Set IN_SEQ to indicate if the REQUESTED_FILE is in sequence     ;
      %*       If the current sequence comparison integer has not been set yet ;
      %*          AND                                                          ;
      %*       The IN_SEQ indicator is set:                                    ;
      %*          Set the current event total indicator to 1.                  ;
      %*          Set the curr sequence comparison integer to SEQUENCE -- the  ;
      %*             req number in the session where the current event occurs. ;
      %*          Set the current funnel indicator to 1.                       ;
      %*       Else:                                                           ;
      %*          Set the current funnel indicator to 0 to indicate that the   ;
      %*             string of contiguous funnel elements has been broken      ;
      else do;
         if &&event&curr_lvl then do;
            level&curr_lvl = 1;
            %*    Set IN_SEQ to indicate if the REQUESTED_FILE is in sequence  ;
            if levelss_&prev_lvl le . then
               in_seq=0;
            else if sequence - levelss_&prev_lvl eq 1 then
               in_seq=1;
            else
               in_seq=0;
            if levelss_&curr_lvl le .z and in_seq then do;
               levelss_&curr_lvl=sequence;
               funnel&curr_lvl = 1;
               curr_hold+1;
            end;
            else do;
               funnel&curr_lvl = 0;
            end;
         end;
      end;
   %end;

   %* If the funnel has contiguous elements:                                   ;
   %*    Determine whether the funnel has a break in contiguity                ;
   if upcase("&contig") eq 'TRUE' then do;
      fun_cnt=0;
      lss_cnt=0;
      do i=1 to curr_hold;
         fun_cnt+funnel{i};
         if levelss{i} gt . then
            lss_cnt+1;
      end;
      if fun_cnt ne lss_cnt then do;
         %* There has been a break in contiguity:                              ;
         %* Dump the active funnel storage area into HOLD                      ;
         if curr_hold gt max_hold then do;
            do j=1 to curr_hold;
               hold_lvl{j}=1;
               hold_fun{j}=1;
               hold_lss{j}=levelss{j};
            end;
            max_hold=curr_hold;
         end;
         %* Re-initialize the active funnel storage area                       ;
         do i=1 to &total_events;
            %* LEVELx is not re-initialized -- it indicates any encounter      ;
            funnel{i}  =0;
            levelss{i} =.;
         end;
      end;
   end;

   if last.session_id then do;
      if upcase("&contig") eq 'TRUE' then do;
         %* move the active funnel storage area to HOLD, if necessary          ;
         if curr_hold gt max_hold then do;
            do j=1 to curr_hold;
               hold_lvl{j}=level{j};
               hold_fun{j}=funnel{j};
               hold_lss{j}=levelss{j};
            end;
            max_hold=curr_hold;
         end;
         %* write out the sessions most complete encounter with the funnel     ;
         do i=1 to &total_events;
            %* LEVELx is not carried over -- it indicates any encounter        ;
            funnel{i}  =hold_fun{i};
            levelss{i} =hold_lss{i};
         end;
      end;
      output;
   end;

   drop i
        j
        hold_lvl1-hold_lvl&total_events
        hold_fun1-hold_fun&total_events
        hold_lss1-hold_lss&total_events
        max_hold
        curr_hold
        fun_cnt
        lss_cnt
        in_seq
       ;

run;
%end;

%* prepare the level-oriented summaries                                        ;
proc summary data = in_f&_funnel_data_uid_ sum;
   var level1-level&total_events;
   output out=fun_f&_funnel_data_uid_.1
           (drop=_type_
                 _freq_)
           sum=;
run;
proc transpose data=fun_f&_funnel_data_uid_.1
               out =xpose_f&_funnel_data_uid_.1
                 (rename=(col1=sessions));
run;

%* prepare the funnel-oriented summaries                                       ;
proc summary data = in_f&_funnel_data_uid_ sum;
   var funnel1-funnel&total_events;
   output out=fun_f&_funnel_data_uid_.2
           (drop=_type_
                 _freq_)
           sum=;
run;
proc transpose data=fun_f&_funnel_data_uid_.2
               out =xpose_f&_funnel_data_uid_.2
                 (rename=(col1=sessions2));
run;

%* combine the level- and funnel-oriented summaries                            ;
data fun_f&_funnel_data_uid_;
   merge xpose_f&_funnel_data_uid_.2
         xpose_f&_funnel_data_uid_.1;
run;

data _null_;
   set fun_f&_funnel_data_uid_ end=eof;
   call symput(compress("users"||
                        strip(put(_n_,6.)))
              ,strip(put(sessions,10.)));
   if eof then
      call symput("tot",strip(put(_n_,6.)));
run;

data fun_f&_funnel_data_uid_.A;
   set fun_f&_funnel_data_uid_;
   if &users1 gt 0 then
      perc_level1 = (sessions2 / &users1) * 100;
   else
      perc_level1 = 0;
run;

data _null_;
   set fun_f&_funnel_data_uid_.A end=eof;
   call symput(compress("perc"||
                        strip(put(_n_,6.)))
                       ,strip(put(perc_level1,6.2)));
   if eof then
      call symput("tot2",strip(put(_n_,6.)));
run;

data fun_f&_funnel_data_uid_.B;
   merge fun_f&_funnel_data_uid_.A
         d&_funnel_data_uid_ (keep = desc);
run;

data stp.fun_f&_funnel_data_uid_;
   length _name_      $10
          desc        $128
          sessions
          sessions2
          perc_level1
          perc_prev
          dropoff     8;
   format sessions
          sessions2   nlnum8.
          perc_prev
          dropoff
          perc_level1 nlnum6.2;

   set fun_f&_funnel_data_uid_.B;

   retain prev_sess2
          lvl1_sess2;

   if perc_level1 le .z then
      perc_level1 = 0;
   _name_ = upcase(_name_);

   if _n_ eq 1 then do;
      lvl1_sess2=sessions2;
      perc_prev=perc_level1;
      dropoff=0;
   end;
   else do;
      if prev_sess2 eq 0 then do;
         perc_prev=0;
         dropoff=0;
      end;
      else do;
         perc_prev=100*(sessions2/prev_sess2);
         dropoff=100-perc_prev;
      end;
   end;

   prev_sess2=sessions2;
   %* NOTE: embedded DESC single quotes were TRANWRDed into paired single      ;
   %*       quotes earlier in the program. restore them here with TRANWRD.     ;
   desc=tranwrd(desc,"''","'");

   label _name_ = "Process"
         desc = "Event"
         sessions = "Total Visits"
         sessions2 = "Path Visits"
         perc_level1 = "% of First Level"
         perc_prev = "% of Previous Level"
         dropoff = "% of Drop Off"
        ;

   keep _name_
        desc
        sessions
        sessions2
        perc_level1
        perc_prev
        dropoff;

run;


%if &samp gt 0.5 %then %do;
   data stp.fun_f&_funnel_data_uid_;
      set stp.fun_f&_funnel_data_uid_(drop=sessions sessions2);
      label perc_level1 = "Estimated % of First Level"
            perc_prev = "Estimated % of Previous Level"
            dropoff = "Estimated % of Drop Off";
   run;

%end;

%let outdsn=stp.fun_f&_funnel_data_uid_;


%DONE:;
%if %upcase(&info) ne TRUE %then %do;
   proc datasets library=work nolist;
      delete contig_&_funnel_data_uid_;
      delete cntlin_&_funnel_data_uid_;
      delete d&_funnel_data_uid_;
      delete extr_f&_funnel_data_uid_;
      delete fun_f&_funnel_data_uid_;
      delete fun_f&_funnel_data_uid_.1;
      delete fun_f&_funnel_data_uid_.2;
      delete fun_f&_funnel_data_uid_.a;
      delete fun_f&_funnel_data_uid_.b;
      delete in_f&_funnel_data_uid_;
      delete meta&_funnel_data_uid_;
      delete xpose_f&_funnel_data_uid_.1;
      delete xpose_f&_funnel_data_uid_.2;
   quit;
%end;


%put *****OUTPUT PARAMETERS*****;
%put stp:output_dataset=&outdsn;
%let     output_dataset=&outdsn;

%*IF &ABORT %THEN %PUT error: The WAFUNNEL macro ended abnormally.             ;
%mend wafunnel;
