/*----------------------Web Analytics-----------------------------------------*/
/* WAPATHDP                                                                   */
/* Assign page IDs, create Path-related data sets                             */
/*                                                                            */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Supported By: Caroline Bahler, CABAHL; Frank Roediger, FRROED              */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* History:                                                                   */
/* Date          Description                    Username         Change Code  */
/* 20041213      Module Created                 cabahl              NA        */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Dependencies:                                                              */
/*   summary.daily_total_day                                                  */
/*                                                                            */
/*                                                                            */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Parameters:                                                                */
/*    retcode  = mechanism for passing Return Codes                           */
/*----------------------------------------------------------------------------*/
/* Input:                                                                     */
/*   summary.                                                                 */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Output:                                                                    */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Copyright (c) 2004 SAS Institute Inc. All Rights Reserved                  */
/*----------------------------------------------------------------------------*/

%macro wapathdp(retcode=
               );

   %let wab_rc = 0;

     %******************************************************************************;
     %* Create pathing data set to be used within interactive pathing and funnels,  ;
     %* and subset to for entry and referrer paths                                  ;
     %******************************************************************************;

     %******************************************************************************;
     %* Replace data for which detail in the wab warehouse has been updated.        ;
     %* New dates will be appended to the pathing data sets for the following       ;
     %* time frames:                                                                ;
     %*                                                                             ;
     %*             most recent day, most recent 7 days, and most recent 30         ;
     %*             days.                                                           ;
     %* This process will depend on the creation of summary.daily_totals_day.       ;
     %******************************************************************************;

     %******************************************************************************;
     %* Determine the data set which contains the most recent date processed.       ;
     %******************************************************************************;

     %let wab_datecheck_dsn=&temp_lib..all_dates;

     %if %sysfunc(exist(summary.daily_totals_day))
        %then %do;

        %if %get_observation_count(indsn=summary.daily_totals_day)=0
           %then %let wab_datecheck_dsn=&temp_lib..all_dates;

        %else %let wab_datecheck_dsn=summary.daily_totals_day;

     %end;

     proc summary data=&wab_datecheck_dsn max;
        var date;
        output out=&temp_lib..max_date(keep=max_date) max=max_date;
     run;

     data _null_;
        set &temp_lib..max_date;
        call symput("_wab_max_date_",max_date);
     run;

     %******************************************************************************;
     %* Build the temporary data sets containing the top entry, and referrer pathing;
     %* variables.                                                                  ;
     %******************************************************************************;

     data &temp_lib..pathing(Keep=Date
                                  Datetime
                                  Session_ID
                                  Sequence
                                  Requested_File
                                  Referrer
                                  Sequenced_Requested_File
                                  Entry_Point
                             where=(date ge (&_wab_max_date_
                                            - &wab_pathing_days_in_history)))

          &temp_lib..referrer_input(keep=Date
                                         Datetime
                                         Session_ID
                                         Sequence
                                         Requested_File
                                         Referrer
                                         Referrer_URL
                                         Entry_Point
                                    rename=(Referrer_URL=Sequenced_Requested_File)
                                    where=(date ge (&_wab_max_date_
                                                   - &wab_pathing_days_in_history)));
        set
            %do wab_iter=1 %to &_wab_num_dates_;
               &temp_lib..detail_&&_wab_date_&wab_iter(keep=Date
                                                            Datetime
                                                            Session_ID
                                                            Requested_File
                                                            Referrer
                                                            Entry_Point
                                                      where=(date ge (&_wab_max_date_
                                                                     - &wab_pathing_days_in_history)))
            %end; ;
        by session_id;

        if first.session_id then
           sequence=.;

        /*----------------------------------------------*
        * output referrer information
        *----------------------------------------------*/
        if Entry_Point=1 then do;
            Sequence=0;
            Referrer_URL=strip(strip(Referrer)
                      || '_'
                      || strip(compress(put(Sequence,9.))));
            output &temp_lib..referrer_input;
            Sequence=1;
        end;
        else if sequence gt . then
           sequence+1;

        if sequence gt .;


        Sequenced_Requested_File=strip(strip(Requested_File)
                              || '_'
                              || strip(compress(put(Sequence,9.))));
        /*----------------------------------------------*
        * output path information
        *----------------------------------------------*/
        output &temp_lib..pathing;

     run;


     %******************************************************************************;
     %* Append the referrer information back on to the temporary pathing input data ;
     %* and sort it.                                                                ;
     %******************************************************************************;
     %if %sysfunc(exist(summary.pathing)) %then %do;
        /*-------------------------------------------------*
        * Append new pathing data to existing pathing
        * data that has old dates aged off the data set
        *-------------------------------------------------*/
        data &temp_lib..pathing;
           set &temp_lib..referrer_input
               &temp_lib..pathing
               summary.pathing(where=(date ge (&_wab_max_date_
                                              - &wab_pathing_days_in_history))
                               drop=reference sequenced_reference);
        run;
     %end;
     %else %do;
        data &temp_lib..pathing;
           set &temp_lib..referrer_input
               &temp_lib..pathing;
        run;
     %end;

     %******************************************************************************;
     %* Recreate summary.pathing                                                    ;
     %******************************************************************************;
     proc sort data=&temp_lib..pathing
               out =summary.pathing nodupkey;
        by Session_id
           Sequence;
     run;

     %if &syserr gt 4 %then %do;
         %put %unquote(&wab_error) PROC SORT to sequence the pathing %QCMPRES(
               history) data in correct order failed.;
         %goto ERREXIT;
     %end;


     %******************************************************************************;
     %*  Create a format to translate REQUESTED_FILEs into a numeric ID and         ;
     %*     a companion format to translate the numeric IDs back into their         ;
     %*     corresponding REQUESTED_FILEs.                                          ;
     %******************************************************************************;
     proc summary data=summary.pathing
                    (keep=sequenced_requested_file
                          requested_file session_id)
                  missing chartype sum;
        class requested_file
              sequenced_requested_file
              session_id;

        types sequenced_requested_file
              requested_file*session_id
              ;

        output out=&temp_lib.._wab_seq_urls (
               where=(sequenced_requested_file ne '')
               drop=_freq_ _type_
              );

        output out=&temp_lib.._wab_page_sessions(
             where=(session_id ne '' and requested_file ne '')
               drop=_freq_ _type_
               );
     run;

     proc summary data=&temp_lib.._wab_page_sessions nway missing chartype sum;
   class requested_file;
   output out=&temp_lib.._wab_urls (rename=(_freq_=session_hit_count)
                                            drop=_type_);
     run;

     data &temp_lib..cntlin_urltoind_(keep=label start fmtname)
          summary.page_frequencies(keep=label session_hit_count fmtname start id);
        length start $1024
               label $1024;
        attrib id
            length  =$36
            format  =$36.
            informat=$36.
            label='ID for metadata maintenance';

        set &temp_lib.._wab_urls;

        length fmtname $ 8;

        index+1;

        /*------------------------------------------------*
        * Format to convert requested_file value to
        * index number
        *------------------------------------------------*/
        start=requested_file;
        label=strip(put(index,12.));
        fmtname='$url2idx';
        output &temp_lib..cntlin_urltoind_;
        /*------------------------------------------------*
        * Format to convert index number to requested_file
        * value.
        *------------------------------------------------*/
        start=strip(put(index,12.));
        id=start;
        label=requested_file;
        fmtname='$idxtorf';
        output summary.page_frequencies;

     run;

     %if &syserr gt 4 %then %do;
       %put %unquote(&wab_error) PAGE_FREQUENCIES cannot be written %QCMPRES(to
         the) SUMMARY library.;
       %put %unquote(&wab_error) REQUESTED_FILEs cannot be created %QCMPRES(for
         Interactive) Funnel.;
       %goto ERREXIT;
     %end;

     proc datasets library=summary nolist;
      modify page_frequencies;
      label label='Requested File';
      index create id
             / nomiss
               unique;
     quit;

     proc format cntlin=&temp_lib..cntlin_urltoind_;
     run;

     proc format cntlin=summary.page_frequencies;
     run;

     data &temp_lib..cntlin_sequrltoind_
          &temp_lib..cntlin_indtosequrl_;
        length start $1037
               label $1037;

        set &temp_lib.._wab_seq_urls;

        length fmtname $ 8;

        index+1;

        start=sequenced_requested_file;
        label=strip(put(index,12.));
        fmtname='$srl2idx';
        output &temp_lib..cntlin_sequrltoind_;

        start=strip(put(index,12.));
        label=sequenced_requested_file;
        fmtname='$idx2srl';
        output &temp_lib..cntlin_indtosequrl_;

     run;

     proc format cntlin=&temp_lib..cntlin_sequrltoind_;
     run;

     proc format cntlin=&temp_lib..cntlin_indtosequrl_;
     run;

     data summary.pathing;
        length reference sequenced_reference $ 12;
        set summary.pathing;
        reference=strip(put(requested_file,$url2idx.));
        sequenced_reference=strip(put(sequenced_requested_file,$srl2idx.));
     run;

     %******************************************************************************;
     %* Build the DMDB catalogs and run PROC PATH to generate the frequent sequences;
     %* for the most recent day, most recent 7 days, and most recent 30 days in the ;
     %* warehouse. Support counts are initialized to 10,40, and 100 respectively.   ;
     %* The maximum number of items in a sequence is 7.                             ;
     %******************************************************************************;

     %******************************************************************************;
     %* Most recent day path summary.                                               ;
     %******************************************************************************;

     PROC DMDB batch data=summary.pathing
                     dmdbcat=&temp_lib..pathing_dmdbcat
                     out=summary.pathing_dmdb
                     normlen=1024;

        ID Session_ID Sequence Date;
        CLASS Sequenced_Reference Reference;
        TARGET Sequenced_Reference Reference;
     RUN;

     PROC PATH dmdbcat=&temp_lib..pathing_dmdbcat
               data=summary.pathing_dmdb(keep=Session_ID Sequence Date Reference
                                         where=((sequence gt 0)
                                                  and
                                           (date=&_wab_max_date_)))
               out=summary.paths_r1(drop=rule)
               support=&wab_r1_support
               items=&wab_itemset_size;

         customer Session_Id;
         sequence Sequence;
         target Reference;
     run;

     data summary.paths_r1(drop=item_new i);

        length item_new $ 1024 item1-item&wab_itemset_size $ 1024;
        array item {1:&wab_itemset_size} $ item1-item&wab_itemset_size;
        format date date9.;

        set summary.paths_r1;

        date=&_wab_max_date_;

        do i=1 to &wab_itemset_size;

           item_new=put(strip(item{i}),$idxtorf.);
           item{i}=item_new;

        end;

      %wametatr(srce_dsn=paths
               ,retcode=wab_rc
               );

     run;

     %if &syserr gt 4 %then %do;
        %put %unquote(&wab_error) The 1 day pathing %QCMPRES(
             data) set SUMMARY.PATHS_R1 set could not be created.;
        %goto ERREXIT;
     %end;

     %******************************************************************************;
     %* Most recent 7 days path summary.                                            ;
     %******************************************************************************;

     PROC PATH dmdbcat=&temp_lib..pathing_dmdbcat
               data=summary.pathing_dmdb(keep=Session_ID Sequence Date Reference
                                    where=((sequence gt 0)
                                                  and
                                           (date ge &_wab_max_date_-7)))
               out=summary.paths_r7(drop=rule)
               support=&wab_r7_support
               items=&wab_itemset_size;

         customer Session_Id;
         sequence Sequence;
         target Reference;

     run;

     data summary.paths_r7(drop=item_new i);

        length item_new $ 1024 item1-item&wab_itemset_size $ 1024;
        array item {1:&wab_itemset_size} $ item1-item&wab_itemset_size;

        format date date9.;

        set summary.paths_r7;

        date=&_wab_max_date_;

        do i=1 to &wab_itemset_size;

           item_new=put(strip(item{i}),$idxtorf.);
           item{i}=item_new;

        end;

      %wametatr(srce_dsn=paths
               ,retcode=wab_rc
               );

     run;

     %if &syserr gt 4 %then %do;
        %put %unquote(&wab_error) The rolling 7 day pathing %QCMPRES(
             data) set SUMMARY.PATHS_R7 set could not be created.;
        %goto ERREXIT;
     %end;

     %******************************************************************************;
     %* Most recent 30 days path summary.                                           ;
     %******************************************************************************;

     PROC PATH dmdbcat=&temp_lib..pathing_dmdbcat
               data=summary.pathing_dmdb(keep=Session_ID Sequence Date Reference
                                    where=(sequence gt 0))
               out=summary.paths_r30(drop=rule)
               support=&wab_r30_support
               items=&wab_itemset_size;

         customer Session_Id;
         sequence Sequence;
         target Reference;
     run;

     data summary.paths_r30(drop=item_new i);

        length item_new $ 1024 item1-item&wab_itemset_size $ 1024;
        array item {1:&wab_itemset_size} $ item1-item&wab_itemset_size;

        format date date9.;

        set summary.paths_r30;

        date=&_wab_max_date_;

        do i=1 to &wab_itemset_size;

           item_new=put(strip(item{i}),$idxtorf.);
           item{i}=item_new;

        end;

      %wametatr(srce_dsn=paths
               ,retcode=wab_rc
               );

     run;

     %if &syserr gt 4 %then %do;
        %put %unquote(&wab_error) The rolling 30 day pathing %QCMPRES(
             data) set SUMMARY.PATHS_R30 set could not be created.;
        %goto ERREXIT;
     %end;

     %******************************************************************************;
     %* Most recent day top entry path summary.                                     ;
     %******************************************************************************;

     PROC PATH dmdbcat=&temp_lib..pathing_dmdbcat
               data=summary.pathing_dmdb(keep=Session_ID Sequence Date Sequenced_Reference
                                    where=((sequence gt 0)
                                                  and
                                           (date=&_wab_max_date_)))
               out=summary.top_entry_paths_r1(drop=rule)
               support=&wab_r1_support
               items=&wab_itemset_size;

         customer Session_Id;
         sequence Sequence;
         target Sequenced_Reference;
     run;

     %******************************************************************************;
     %* Parse the sequence indicators off of the items in a sequence. Keep only     ;
     %* entry paths.                                                                ;
     %******************************************************************************;

     data summary.top_entry_paths_r1(drop=i npos item_new item_new_1);
        length item_new $ 1037 item1-item&wab_itemset_size $ 1037;
        array item {1:&wab_itemset_size} $ item1-item&wab_itemset_size;
        format date date9.;
        set summary.top_entry_paths_r1;
        date=&_wab_max_date_;

        do i=1 to &wab_itemset_size;

           item_new=put(strip(item{i}),$idx2srl.);

           if item_new ne " " then do;

              if i=1 then item_new_1=item_new;

              npos=prxmatch('/_\d{1,2}$/',strip(item_new))-1;

              if npos ge 1 then do;

                 item_new=substr(strip(item_new)
                                ,1
                                ,npos);

                 item{i}=item_new;

              end;

              else do;

                 item{i}=item_new;

              end;

           end;

        end;

        if prxmatch('/_1$/',strip(item_new_1));

      %wametatr(srce_dsn=paths
               ,retcode=wab_rc
               );

     run;

     %if &syserr gt 4 %then %do;
        %put %unquote(&wab_error) The daily top entry pathing %QCMPRES(
             data) set SUMMARY.TOP_ENTRY_PATHS_R1 set could not be created.;
        %goto ERREXIT;
     %end;

     %******************************************************************************;
     %* Most recent 7 days top entry path summary.                                  ;
     %******************************************************************************;

     PROC PATH dmdbcat=&temp_lib..pathing_dmdbcat
               data=summary.pathing_dmdb(keep=Session_ID Sequence Date Sequenced_Reference
                                    where=((sequence gt 0)
                                                  and
                                           (date ge &_wab_max_date_-7)))
               out=summary.top_entry_paths_r7(drop=rule)
               support=&wab_r7_support
               items=&wab_itemset_size;

         customer Session_Id;
         sequence Sequence;
         target Sequenced_Reference;

     run;

     %******************************************************************************;
     %* Parse the sequence indicators off of the items in a sequence. Keep only     ;
     %* entry paths.                                                                ;
     %******************************************************************************;

     data summary.top_entry_paths_r7(drop=i npos item_new item_new_1);
        length item_new $ 1037 item1-item&wab_itemset_size $ 1037;
        array item {1:&wab_itemset_size} $ item1-item&wab_itemset_size;
        format date date9.;
        set summary.top_entry_paths_r7;
        date=&_wab_max_date_;

        do i=1 to &wab_itemset_size;

           item_new=put(strip(item{i}),$idx2srl.);

           if item_new ne " " then do;

              if i=1 then item_new_1=item_new;

              npos=prxmatch('/_\d{1,2}$/',strip(item_new))-1;

              if npos ge 1 then do;

                 item_new=substr(strip(item_new)
                                ,1
                                ,npos);

                 item{i}=item_new;

              end;

              else do;

                  item{i}=item_new;

              end;

           end;

        end;

        if prxmatch('/_1$/',strip(item_new_1));

      %wametatr(srce_dsn=paths
               ,retcode=wab_rc
               );

     run;

     %if &syserr gt 4 %then %do;
        %put %unquote(&wab_error) The rolling 7 day top entry pathing %QCMPRES(
             data) set SUMMARY.TOP_ENTRY_PATHS_R7 set could not be created.;
        %goto ERREXIT;
     %end;

     %******************************************************************************;
     %* Most recent 30 days top entry path summary.                                 ;
     %******************************************************************************;

     PROC PATH dmdbcat=&temp_lib..pathing_dmdbcat
               data=summary.pathing_dmdb(keep=Session_ID Sequence Date Sequenced_Reference
                                         where=((sequence gt 0)
                                                 and
                                                (date ge &_wab_max_date_- 30))
                                         )
               out=summary.top_entry_paths_r30(drop=rule)
               support=&wab_r30_support
               items=&wab_itemset_size;

         customer Session_Id;
         sequence Sequence;
         target Sequenced_Reference;

     run;


     %******************************************************************************;
     %* Parse the sequence indicators off of the items in a sequence. Keep only     ;
     %* entry paths.                                                                ;
     %******************************************************************************;

     data summary.top_entry_paths_r30(drop=i npos item_new item_new_1);

        length item_new $ 1037 item1-item&wab_itemset_size $ 1037;
        array item {1:&wab_itemset_size} $ item1-item&wab_itemset_size;
        format date date9.;
        set summary.top_entry_paths_r30;
        date=&_wab_max_date_;

        do i=1 to &wab_itemset_size;

           item_new=put(strip(item{i}),$idx2srl.);

           if item_new ne " " then do;

              if i=1 then item_new_1=item_new;

              npos=prxmatch('/_\d{1,2}$/',strip(item_new))-1;

              if npos ge 1 then do;

                 item_new=substr(strip(item_new)
                                ,1
                                ,npos);

                 item{i}=item_new;

              end;

              else do;

                  item{i}=item_new;

              end;

           end;

        end;

        if prxmatch('/_1$/',strip(item_new_1));

      %wametatr(srce_dsn=paths
               ,retcode=wab_rc
               );

     run;

     %if &syserr gt 4 %then %do;
        %put %unquote(&wab_error) The rolling 30 day top entry pathing %QCMPRES(
             data) set SUMMARY.TOP_ENTRY_PATHS_R30 set could not be created.;
        %goto ERREXIT;
     %end;

     %******************************************************************************;
     %* Most recent day referrer path summary.                                      ;
     %******************************************************************************;

     PROC PATH dmdbcat=&temp_lib..pathing_dmdbcat
               data=summary.pathing_dmdb(keep=Session_ID Sequence Date Sequenced_Reference
                                    where=( (date=&_wab_max_date_)
                                          and (sequence ge 0) ) )
               out=summary.top_referrer_paths_r1(drop=rule)
               support=&wab_r1_support
               items=&wab_itemset_size;

         customer Session_Id;
         sequence Sequence;
         target Sequenced_Reference;
     run;


     %******************************************************************************;
     %* Parse the sequence indicators off of the items in a sequence. Keep only     ;
     %* entry paths.                                                                ;
     %******************************************************************************;

     data summary.top_referrer_paths_r1(drop=i npos item_new item_new_1);
        length item_new $ 1037 item1-item&wab_itemset_size $ 1037;
        array item {1:&wab_itemset_size} $ item1-item&wab_itemset_size;
        format date date9.;
        set summary.top_referrer_paths_r1;
        date=&_wab_max_date_;

        do i=1 to &wab_itemset_size;

           item_new=put(strip(item{i}),$idx2srl.);

           if item_new ne " " then do;

              if i=1 then item_new_1=item_new;

              npos=prxmatch('/_\d{1,2}$/',strip(item_new))-1;

              if npos ge 1 then do;

                 item_new=substr(strip(item_new)
                                ,1
                                ,npos);

                 item{i}=item_new;

              end;

              else do;

                  item{i}=item_new;

              end;

           end;


        end;

        if prxmatch('/_0$/',strip(item_new_1));

      %wametatr(srce_dsn=paths
               ,retcode=wab_rc
               );

     run;

     %if &syserr gt 4 %then %do;
        %put %unquote(&wab_error) The daily top referrer pathing %QCMPRES(
             data) set SUMMARY.TOP_REFERRER_PATHS_R1 set could not be created.;
        %goto ERREXIT;
     %end;

     %******************************************************************************;
     %* Most recent 7 day referrer path summary.                                    ;
     %******************************************************************************;

     PROC PATH dmdbcat=&temp_lib..pathing_dmdbcat
               data=summary.pathing_dmdb(keep=Session_ID Sequence Date Sequenced_Reference
                                    where=( (date ge &_wab_max_date_-7)
                                           and (sequence ge 0) ) )
               out=summary.top_referrer_paths_r7(drop=rule)
               support=&wab_r7_support
               items=&wab_itemset_size;

         customer Session_Id;
         sequence Sequence;
         target Sequenced_Reference;

     run;

     %******************************************************************************;
     %* Parse the sequence indicators off of the items in a sequence. Keep only     ;
     %* entry paths.                                                                ;
     %******************************************************************************;

     data summary.top_referrer_paths_r7(drop=i npos item_new item_new_1);
        length item_new $ 1037 item1-item&wab_itemset_size $ 1037;
        array item {1:&wab_itemset_size} $ item1-item&wab_itemset_size;
        format date date9.;
        set summary.top_referrer_paths_r7;
        date=&_wab_max_date_;

        do i=1 to &wab_itemset_size;

           item_new=put(strip(item{i}),$idx2srl.);

           if item_new ne " " then do;

              if i=1 then item_new_1=item_new;

              npos=prxmatch('/_\d{1,2}$/',strip(item_new))-1;

              if npos ge 1 then do;

                 item_new=substr(strip(item_new)
                                ,1
                                ,npos);

                 item{i}=item_new;

              end;

              else do;

                  item{i}=item_new;

              end;

           end;

        end;

        if prxmatch('/_0$/',strip(item_new_1));

      %wametatr(srce_dsn=paths
               ,retcode=wab_rc
               );

     run;

     %if &syserr gt 4 %then %do;
        %put %unquote(&wab_error) The rolling 7 day top referrer pathing %QCMPRES(
             data) set SUMMARY.TOP_REFERRER_PATHS_R7 set could not be created.;
        %goto ERREXIT;
     %end;

     %******************************************************************************;
     %* Most recent day referrer entry path summary.                                ;
     %******************************************************************************;

     PROC PATH dmdbcat=&temp_lib..pathing_dmdbcat
               data=summary.pathing_dmdb(where=(sequence ge 0)
                                    keep=Session_ID Sequence Date Sequenced_Reference)
               out=summary.top_referrer_paths_r30(drop=rule)
               support=&wab_r30_support
               items=&wab_itemset_size;

         customer Session_Id;
         sequence Sequence;
         target Sequenced_Reference;

     run;

     %******************************************************************************;
     %* Parse the sequence indicators off of the items in a sequence. Keep only     ;
     %* entry paths.                                                                ;
     %******************************************************************************;

     data summary.top_referrer_paths_r30(drop=i npos item_new item_new_1);
        length item_new $ 1037 item1-item&wab_itemset_size $ 1037;
        array item {1:&wab_itemset_size} $ item1-item&wab_itemset_size;
        format date date9.;
        set summary.top_referrer_paths_r30;
        date=&_wab_max_date_;

        do i=1 to &wab_itemset_size;

           item_new=put(strip(item{i}),$idx2srl.);

           if item_new ne " " then do;

              if i=1 then item_new_1=item_new;

              npos=prxmatch('/_\d{1,2}$/',strip(item_new))-1;

              if npos ge 1 then do;

                 item_new=substr(strip(item_new)
                                ,1
                                ,npos);

                 item{i}=item_new;

              end;

              else do;

                  item{i}=item_new;

              end;

           end;

        end;

        if prxmatch('/_0$/',strip(item_new_1));

      %wametatr(srce_dsn=paths
               ,retcode=wab_rc
               );

     run;

     %if &syserr gt 4 %then %do;
        %put %unquote(&wab_error) The rolling 30 day top referrer pathing %QCMPRES(
             data) set SUMMARY.TOP_REFERRER_PATHS_R30 set could not be created.;
        %goto ERREXIT;
     %end;


     %******************************************************************************;
     %* Create lists of path entry and ending points                                ;
     %******************************************************************************;
     proc dmdb batch data=summary.pathing
                           (where=((sequence gt 0)
                                    and
                                   (date ge &_wab_max_date_- 30))
                           )
                     dmdbcat=&temp_lib..dmdbcat
                     out=&temp_lib..dmdbdsn
                     normlen=12;
        id     session_id
               sequence
               date;
        class  reference;
        target reference;
     run;

     proc path dmdbcat=&temp_lib..dmdbcat
               data=&temp_lib..dmdbdsn
                       (keep=session_id sequence date reference)
               out=&temp_lib..proc_path_out
               support=&wab_interactive_support
               items=2;
         customer session_id;
         sequence sequence;
         target reference;
     run;


     %******************************************************************************;
     %* create the final lists of pathing start pages (ITEM1s)                      ;
     %******************************************************************************;
     proc sort data=&temp_lib..proc_path_out
                 (keep=item1)
               out =&temp_lib..uniq_start_list
               nodupkeys;
        by item1;
     run;

     data &temp_lib..path_start_list;
        set &temp_lib..uniq_start_list;
        length requested_file $1024;
        index=input(item1,12.);
        requested_file=put(strip(item1),$idxtorf.);
        keep index
             requested_file;
     run;

     proc sort data=&temp_lib..path_start_list
               out =summary.path_start_list;
        by requested_file;
     run;


     %******************************************************************************;
     %* create the final lists of pathing end pages (ITEM2s)                        ;
     %******************************************************************************;
     proc sort data=&temp_lib..proc_path_out
                 (keep=item2)
               out =&temp_lib..uniq_end_list
               nodupkeys;
        by item2;
     run;

     data &temp_lib..path_end_list;
        set &temp_lib..uniq_end_list;
        length requested_file $1024;
        index=input(item2,12.);
        requested_file=put(strip(item2),$idxtorf.);
        keep index
             requested_file;
     run;
     proc sort data=&temp_lib..path_end_list
               out =summary.path_end_list;
        by requested_file;
     run;

  %ERREXIT:
%mend wapathdp;
