%******************************************************************************;
%* Copyright (C) 2004 by SAS Institute Inc., Cary, NC 27512-8000               ;
%*                                                                             ;
%* Name:     Wautils.sas                                                       ;
%*                                                                             ;
%* Purpose:  Bundle the Web Analytics Utility macros:                          ;
%*                                                                             ;
%*             No_Notes                                                        ;
%*             Maybe_Notes                                                     ;
%*             makenulldata                                                    ;
%*             wametatr                                                        ;
%*             watstdsn                                                        ;
%*             Current_OS                                                      ;
%*             make_directory                                                  ;
%*             remove_directory                                                ;
%*             get_observation_count                                           ;
%*             locktest                                                        ;
%*             wametdef                                                        ;
%*             wasasdir                                                        ;
%*             walists                                                         ;
%*             cvt_fmts                                                        ;
%*                                                                             ;
%* Parameters:                                                                 ;
%*                                                                             ;
%*                                                                             ;
%******************************************************************************;

%macro wautils;

%******************************************************************************;
%* Name:     No_Notes                                                          ;
%* Purpose:  Turn off options notes based on wbdebug flag                      ;
%* Usage:    %No_Notes                                                         ;
%******************************************************************************;
%macro No_Notes;

      options nonotes;

%mend No_Notes;


%******************************************************************************;
%* Name:     Maybe_Notes                                                       ;
%* Purpose:  Turn on notes only if mode is not Quiet                           ;
%* Usage:    %Maybe_Notes                                                      ;
%******************************************************************************;
%macro Maybe_Notes;

   options notes;

%mend Maybe_Notes;


/*-----------------------------------------------------------------------------*/
/*     MAKENULLDATA                                                            */
/*                                                                             */
/*     HISTORY: 09/01/04 sassyw  Initial coding                                */
/*                                                                             */
/*-----------------------------------------------------------------------------*/
/*     DESCRIPTION:   This macro is used to generate a template data set.      */
/*                    The data set will have no records only the header info.  */
/*                    This macro was written to centralize the attributes of   */
/*                    the data sets which it generates.                        */
/*                                                                             */
/*     PARAMETERS:    DSTYPE    The type of data set to generate.  Also could  */
/*                              be stated what process needs the null data     */
/*                              set generated.  Valid values are SCORECARD and */
/*                              DASHBOARD                                      */
/*                    DSLIB     The library to create the data set in          */
/*                    DSNAME    The name to use for the dataset                */
/*                    REPLACE   Flag indicating if an existing version of      */
/*                              the data set should be replaced or not         */
/*                              (Default=YES)                                  */
/*-----------------------------------------------------------------------------*/
%macro makenulldata(dstype=
                    ,dslib=
                    ,dsname=
                    ,replace=YES);


/* normalize all parameters to upper case */
%let dstype=%upcase(&dstype);
%let dslib=%upcase(&dslib);
%let dsname=%upcase(&dsname);
%let replace=%upcase(&replace);

%let valid_types=SCORECARD DASHBOARD SEGMENTATION;
%let valid_replace=YES NO;

/* verify that the libref is defined */
%if (%sysfunc(libref(&dslib))) %then %do;
   %PUT %UNQUOTE(&wab_error) Library &dslib specified in DSLIB parameter is not defined;
   %LET retcode=1;
   %GOTO ERREXIT;
%end;

/* verify that the dstype is one of SCORECARD DASHBOARD SEGMENTATION */
%if (%sysfunc(index(&valid_types,&dstype)))=0  %then %do;
   %PUT %UNQUOTE(&wab_error) Type &dstype specified in DSTYPE parameter is not valid;
   %PUT %UNQUOTE(&wab_error) Valid values are SCORECARD, DASHBOARD or SEGMENTATION;
   %LET retcode=1;
   %GOTO ERREXIT;
%end;

/* verify that the replace parameter value is valid */
%if (%sysfunc(index(&valid_replace,&replace)))=0  %then %do;
   %PUT %UNQUOTE(&wab_error) Valid values for the REPLACE parameter are &valid_replace;
   %LET retcode=1;
   %GOTO ERREXIT;
%end;

/* build the full libname.dsname name of the data set to be created */
%let fullname=&dslib..&dsname;

/* delete if the dataset exist and the replace parameter value is YES */
%IF %SYSFUNC(EXIST(&fullname,DATA)) and (&replace=NO) %THEN %DO;
   %PUT %UNQUOTE(&wab_error) The data set &fullname already exist;
   %PUT %UNQUOTE(&wab_error) The REPLACE parameter is set to &replace;
   %LET retcode=1;
   %GOTO ERREXIT;
%END;
%ELSE %DO;
   %IF %SYSFUNC(EXIST(&fullname,DATA))%THEN %DO;
      proc datasets lib=&dslib nolist;
         delete &dsname;
      run;
   %END;
%END;


%IF &dstype=SCORECARD %THEN %DO;
   /* ready to build scorecard table with null record */

   data &fullname;
     attrib scorecard
               length  =$15
            report_date
               length  =8
               format  =DATE7.
            _LABEL_
               length  =$40
               format  =$100.
               informat=$100.
               label='LABEL OF FORMER VARIABLE'
            _0
               length  =8
            _5
               length  =8
            _10
               length  =8
            _15
               length  =8
            _20
               length  =8
            _25
               length  =8
            _30
               length  =8
            _35
               length  =8
            _40
               length  =8
            _45
               length  =8
            _50
               length  =8
            DATE
               length  =8
               format  =DATE9.
               informat=YYMMDD10.
               label='Date'
            ACTUAL
               length  =8
               label='Actual Values'
            PREDICT
               length  =8
               label='Predicted Values'
            LOWER
               length  =8
               label='Lower Confidence Limits'
            UPPER
               length  =8
               label='Upper Confidence Limits'
            ERROR
               length  =8
               label='Prediction Errors'
            STD
               length  =8
               label='Prediction Standard Errors'
            business_dir
               length  =$8
               format  =$8.
               informat=$8.
            performance
               length  =8
            business_goal
               length  =8
            _WEIGHT_
               length  =8
               format  =nlnum12.2
            importance
               length  =8
            _name_
               length  =$25;
   run;

%END;

%IF &dstype=DASHBOARD %THEN %DO;
   /* ready to build dashboard table with null record */

   data &fullname;
      length slope 8
             slope_stderr 8
             intercept 8
             std_trend 8
             var_name $64
             report_date 8
             dashboard $25
             label $64
             actual 8
             min 8
             max 8
             mean 8
             std 8
             category $45
             business_dir $5
             pvalue 8
             performance $15
            ;
      format report_date date9.
             std_trend nlnum15.2
            ;
   run;


%END;

%IF &dstype=SEGMENTATION %THEN %DO;
    /* ready to build autoseg */
    data &fullname;
      length id $25.;
      format segment_rule_label $2000.
             predicted_0
             predicted_1
             weighted_response  nlnum8.2
             all_visitors       nlnum12.
             pc_visits          nlnum8.2
             exp_resp_all       nlnum12.
             new_visitors       nlnum12.
             pc_visits_new      nlnum8.2
             exp_resp_new       nlnum12.
             maxdate_visitors   nlnum12.
             pc_visits_maxdate  nlnum8.2
             exp_resp_maxdate   nlnum12.
             ;

      label
              all_visitors='All Visitors'
              new_visitors='New Visitors'
              maxdate_visitors='Most Current Visitors'
              pc_visits='% All Visitors'
              pc_visits_new='% New Visitors'
              pc_visits_maxdate='% Most Current Visitors'
              exp_resp_all='All Predicted Responders'
              exp_resp_new='New Predicted Responders'
              exp_resp_maxdate='Most Current Predicted Responders';
    run;
%END;

%ERREXIT:

%mend makenulldata;


%******************************************************************************;
%* Name:     wametatr                                                          ;
%* Purpose:  Assign standard attributes to WA metadata variables.              ;
%*           NOTE: there are two blocks of code -- one for the CONFIG library  ;
%*                 data sets, and a 2nd for the DOCUMENT library data sets     ;
%* Usage:    %wametatr(srce_dsn=wasumeng                                       ;
%*                    ,retcode=wab_rc                                          ;
%*                    )                                                        ;
%* Change History                                                              ;
%* Date        Description                              Username   Change Code ;
%* 11/10/2004  Added label and changed scorecard,       cabahl     S0275612    ;
%*             dashboard and segmentation_name to name in WASCRCRD, WADSHBRD   ;
%*             WAAUTOSG attributes statements                                  ;
%******************************************************************************;
%macro wametatr(srce_dsn=
               ,retcode=
               );

   %local macname;
   %let   macname=&sysmacroname;

   %let &retcode=0;

   %* start of the CONFIG library attributes --------------------------------- ;
   %if %upcase(&srce_dsn) eq WASUMENG %then %do;
      attrib id
               length  =$36
               format  =$36.
               informat=$36.
               label='ID for metadata maintenance'
             input_table
               length  =$41
               format  =$41.
               informat=$41.
               label='LIBNAME and DSN of input data set'
             summary_stmt_opts
               length  =$1024
               format  =$1024.
               informat=$1024.
               label='PROC SUMMARY statement options'
             where_stmt
               length  =$4096
               format  =$4096.
               informat=$4096.
               label='Input data set WHERE clause'
             partition_by_date
               length  =8
               format  =8.
               informat=8.
               label='Indicator: is input/output data date partitioned?'
             id_vars
               length  =$4096
               format  =$4096.
               informat=$4096.
               label='Comma-delimited list of ID vars'
             class_vars
               length  =$4096
               format  =$4096.
               informat=$4096.
               label='Comma-delimited list of Class vars'
             analysis_vars
               length  =$4096
               format  =$4096.
               informat=$4096.
               label='Comma-delimited list of Analysis vars'
             freq_vars
               length  =$4096
               format  =$4096.
               informat=$4096.
               label='Comma-delimited list of Freq vars'
             weight_vars
               length  =$4096
               format  =$4096.
               informat=$4096.
               label='Comma-delimited list of Weight vars'
             output_table
               length  =$41
               format  =$41.
               informat=$41.
               label='LIBNAME and DSN of output data set'
             summary_outdsn_opts
               length  =$4096
               format  =$4096.
               informat=$4096.
               label='OUTPUT OUT= data set options (except for WHERE)'
             summary_outdsn_where
               length  =$4096
               format  =$4096.
               informat=$4096.
               label='OUTPUT OUT= WHERE clause'
             summary_output_opts
               length  =$1024
               format  =$1024.
               informat=$1024.
               label='OUTPUT statement options'
         ;
   %end;
   %else %if %upcase(&srce_dsn) eq WAINIT or
             %upcase(&srce_dsn) eq WBINIT %then %do;
      attrib Name
               length=$32
               label="Webmart Name"
             Desc
               length=$200
               label="Description"
             Perm
               length=$200
               label="Webmart Location"
             Flag
               length=3
               label="Internal use"
         ;
   %end;
   %else %if %upcase(&srce_dsn) eq WADSHINP %then %do;
      attrib id
               length  =$36
               format  =$36.
               informat=$36.
               label='ID for metadata maintenance'
             dashboard_id
               length=$36
               format  =$36.
               informat=$36.
               label='Dashboard ID'
             time_span
               length=$25
               format  =$25.
               informat=$25.
               label='Time Span'
             role
               length=$25
               format  =$25.
               informat=$25.
               label='Variable Role'
             _name_
               length=$32
               format  =$32.
               informat=$32.
               label='Variable Name'
             _name_category_
               length=$32
               format =$32.
               informat=$32.
               label='Variable Category'
             _label_
               length=$100
               format  =$100.
               informat=$100.
               label='Variable Label'
             business_dir
               length=$8
               format  =$8.
               informat=$8.
               label='Positive Business Direction'
             link
               length=$4096
               format  =$4096.
               informat=$4096.
               label='Hyperlink for Customized Drill'
            ;
   %end;
   %else %if %upcase(&srce_dsn) eq WADSHBRD %then %do;
      attrib id
               length  =$36
               format  =$36.
               informat=$36.
               label="Dashboard ID"
             name
               length=$25
               format  =$25.
               informat=$25.
               label="Dashboard Name"
             indsn
               length=$41
               format  =$41.
               informat=$41.
               label="Input Data Set Name"
             label
               length=$25
               format  =$25.
               informat=$25.
               label="Dashboard Label"
            ;
   %end;
   %else %if %upcase(&srce_dsn) eq WASCRINP %then %do;
      attrib id
               length  =$36
               format  =$36.
               informat=$36.
               label='ID for metadata maintenance'
             scorecard_id
               length=$36
               format  =$36.
               informat=$36.
               label="Scorecard ID"
             role
               length=$25
               format  =$25.
               informat=$25.
               label="Variable Role"
             _name_
               length=$32
               format  =$32.
               informat=$32.
               label="Variable Name"
             _label_
               length=$100
               format  =$100.
               informat=$100.
               label="Variable Label"
             business_dir
               length=$8
               format  =$8.
               informat=$8.
               label="Positive Business Direction"
             measurement_range
               length=$40
               format  =$40.
               informat=$40.
               label="Variable Measurement Range"
             link
               length=$4096
               format  =$4096.
               informat=$4096.
               label='Hyperlink for Customized Drill'
            ;
   %end;
   %else %if %upcase(&srce_dsn) eq WASCRCRD %then %do;
      attrib id
               length=$36
               format  =$36.
               informat=$36.
               label="Scorecard ID"
             name
               length=$25
               format  =$25.
               informat=$25.
               label="Scorecard Name"
             indsn
               length=$41
               format  =$41.
               informat=$41.
               label="Input Data Set Name"
             label
               length=$25
               format  =$25.
               informat=$25.
               label="Scorecard Label"
            ;
   %end;
   %else %if %upcase(&srce_dsn) eq WAAUTINP %then %do;
      attrib id
               length  =$36
               format  =$36.
               informat=$36.
               label='ID for metadata maintenance'
           Segmentation_ID
               length=$36
               format  =$36.
               informat=$36.
               label="Segmentation ID"
           role
               length=$25
               format  =$25.
               informat=$25.
               label="Variable Role"
           level
               length=$25
               format=$25.
               informat=$25.
               label="Variable Measurement Level"
           _name_
               length=$32
               format  =$32.
               informat=$32.
               label="Variable Name"
           _label_
               length=$100
               format  =$100.
               informat=$100.
               label="Variable Label"
           ;
   %end;
   %else %if %upcase(&srce_dsn) eq WAAUTOSG %then %do;
      attrib id
               length  =$36
               format  =$36.
               informat=$36.
               label="Segmentation ID"
           name
               length=$25
               format  =$25.
               informat=$25.
               label="Segmentation Name"
           indsn
               length=$41
               format=$41.
               informat=$41.
               label="Input Data Set Name"
           num_segments_to_create
               length  =8
               label='Number of segments to create'
           data_prep_include_code
               length=$2000
               label='Data preparation code'
           label
               length=$25
               format  =$25.
               informat=$25.
               label="Segmentation Label"
           ;
   %end;
   %else %if %upcase(&srce_dsn) eq WACONFIG %then %do;
      attrib id
               length  =$36
               format  =$36.
               informat=$36.
               label='ID for metadata maintenance'
           value
               length=$128
               format  =$128.
               informat=$128.
               label="Value"
           description
               length=$1024
               format=$1024.
               informat=$1024.
               label="Description"
           parameter_name
               length=$32
               format  =$32.
               informat=$32.
               label="Parameter Name"
           ;
   %end;
   %else %if %upcase(&srce_dsn) eq WADTSORT %then %do;
      attrib Dow_id
               length  =$2
               format  =$2.
               informat=$2.
               label='Day of Week ID'
             Dow_Name
               length=$12
               label ="Day of Week Name"
             Dow_Sort
               length  =8
               format  =BEST12.
               informat=BEST12.
               label="Day of Week Sort Value"
               ;
   %end;
   %else %if %upcase(&srce_dsn) eq WARESRCE %then %do;
      attrib id
               length  =$64
               format  =$64.
               informat=$64.
               label='ID -- name of feature'
           value
               length=$1024
               format  =$1024.
               informat=$1024.
               label="Value/Setting of feature"
           description
               length=$256
               format=$256.
               informat=$256.
               label="Description of feature"
           ;
   %end;
   %else %if %upcase(&srce_dsn) eq WATBLDSC %then %do;
      attrib id
               length  =$64
               format  =$64.
               informat=$64.
               label='ID -- name of feature'
           value
               length=$1024
               format  =$1024.
               informat=$1024.
               label="Value/Setting of feature"
           description
               length=$256
               format=$256.
               informat=$256.
               label="Description of feature"
           ;
   %end;
   %else %if %upcase(&srce_dsn) eq WAADMSUM %then %do;
      attrib id
               length  =$36
               format  =$36.
               informat=$36.
               label='ID for metadata maintenance'
           label
               length=$256
               format  =$256.
               informat=$256.
               label="Label to Identify Summary Type"
           source_table
               length=$32
               format  =$32.
               informat=$32.
               label="Name of Source Table"
           proc_summary_options
               length=$1024
               format  =$1024.
               informat=$1024.
               label="PROC SUMMARY options"
           input_where
               length=$1024
               format  =$1024.
               informat=$1024.
               label="WHERE clause for filtering input data set"
           summary_level_list
               length=$30
               format=$30.
               informat=$30.
               label="Comma-delimited list of summary levels for ID vars"
           class_var_list
               length=$4096
               format  =$4096.
               informat=$4096.
               label="Comma-delimited list of CLASS vars"
           id_var_list
               length=$4096
               format  =$4096.
               informat=$4096.
               label="Comma-delimited list of ID vars"
           id_var_summary_level_list
               length=$30
               format=$30.
               informat=$30.
               label="Comma-delimited list of summary levels for ID vars"
           analysis_var_list
               length=$4096
               format  =$4096.
               informat=$4096.
               label="Comma-delimited list of Analysis vars"
           output_dsn_options_0
               length=$4096
               format  =$4096.
               informat=$4096.
               label="Data set options for the re-summarization OUT= data set"
           output_dsn_options_1
               length=$4096
               format  =$4096.
               informat=$4096.
               label="Data set options for the initial summarization OUT= data set"
           output_dsn_where_stmt
               length=$4096
               format  =$4096.
               informat=$4096.
               label="Data set WHERE statement for the OUT= data set"
           output_options
               length=$1024
               format  =$1024.
               informat=$1024.
               label="PROC SUMMARY keyword options"
           results_table
               length=$26
               format  =$26.
               informat=$26.
               label="Name of Table with Summarized Results"
           summary_level_list
               length=$30
               format=$30.
               informat=$30.
               label="Comma-delimited list of summary levels"
           ;
   %end;
   %else %if %upcase(&srce_dsn) eq WAENGVAR %then %do;
      attrib id
               length  =$36
               format  =$36.
               informat=$36.
               label='ID for metadata maintenance'
           libname
               length=$8
               format  =$8.
               informat=$8.
               label="Library Name"
           memname
               length=$32
               format  =$32.
               informat=$32.
               label="Member/DataSet Name"
           name
               length=$32
               format  =$32.
               informat=$32.
               label="Column Name"
           label
               length=$256
               format  =$256.
               informat=$256.
               label="Column Label"
           ;
   %end;
   %else %if %upcase(&srce_dsn) eq WAFUNNEL %then %do;
      attrib id
               length  =$36
               format  =$36.
               informat=$36.
               label='ID for metadata maintenance'
           label
               length=$128
               format  =$128.
               informat=$128.
               label="Funnel Name"
           url
               length=$255
               format  =$255.
               informat=$255.
               label="Comma-delimited list of Web Page/URL IDs"
           adjacent
               length=$255
               format  =$255.
               informat=$255.
               label="Comma-delimited list of Adjacent indicators"
           ;
   %end;
   %else %if %upcase(&srce_dsn) eq WAPATH %then %do;
      attrib id
               length  =$36
               format  =$36.
               informat=$36.
               label='ID for metadata maintenance'
           label
               length=$128
               format  =$128.
               informat=$128.
               label="Path Name"
           start_page
               length=$255
               format  =$255.
               informat=$255.
               label="Comma-delimited list of IDs for REQUESTED_FILE Start Pages"
           end_page
               length=$255
               format  =$255.
               informat=$255.
               label="Comma-delimited list of IDs for REQUESTED_FILE End Pages"
           topn
               length=$5
               format  =$5.
               informat=$5.
               label="Maximum number of rows that will be in the results table"
           sequence_length
               length=$5
               format  =$5.
               informat=$5.
               label="Maximum number of requested files in a path"
           ;
   %end;
   %* end of the CONFIG library attributes ----------------------------------- ;


   %* start of the DOCUMENT library attributes ------------------------------- ;
   %else %if %upcase(&srce_dsn) eq ASSIGNMENT %then %do;
      attrib Id
               length  =$36
             Datasource
               length  =$36
             Libname
               length  =$36
             Path
               length  =$255
         ;
   %end;
   %else %if %upcase(&srce_dsn) eq DATASOURCE %then %do;
      attrib Id
               length  =$36
             Label
               length  =$128
             Documents
               length  =$36
             DataHost
               length  =$128
             DataPort
               length  =8
             DataUser
               length  =$128
             DataPassword
               length  =$128
             ProcessHost
               length  =$128
             ProcessPort
               length  =8
             ProcessUser
               length  =$128
             ProcessPassword
               length  =$128
             Driver
               length  =$128
             Prefix
               length  =$128
             Repository
               length  =$128
         ;
   %end;
   %else %if %upcase(&srce_dsn) eq DEFINITION %then %do;
      attrib Id
               length  =$36
             Label
               length  =$128
             Description
               length  =$255
             DocGroup
               length  =$36
             Helpfile
               length  =$255
             DefaultGraph
               length  =$36
             SortField
               length  =$36
             SortOrder
               length  =8
             Delimiter
               length  =$1
             Sortnum
               length  =8
             IsPublic
               length  =8
             URL
               length  =$1024
         ;
   %end;
   %else %if %upcase(&srce_dsn) eq DOCGROUP %then %do;
      attrib Id
               length  =$36
             Label
               length  =$128
             Description
               length  =$255
             Documents
               length  =$36
             Sortnum
               length  =8
             IsPublic
               length  =8
         ;
   %end;
   %else %if %upcase(&srce_dsn) eq DOCUMENTS %then %do;
      attrib Id
               length  =$36
             Label
               length  =$128
             Description
               length  =$255
             PrimeDatasource
               length  =$36
             GraphStyle
               length  =$36
             GraphScheme
               length  =$36
             GraphWidth
               length  =8
             GraphHeight
               length  =8
         ;
   %end;
   %else %if %upcase(&srce_dsn) eq ENTRY %then %do;
      attrib Id
               length  =$36
             Label
               length  =$128
             Description
               length  =$255
             Parent
               length  =$36
             EntryTag
               length  =$36
             Definition
               length  =$36
             Datasource
               length  =$36
             Db
               length  =$128
             DbTable
               length  =$128
             Program
               length  =$36
             IsAll
               length  =8
         ;
   %end;
   %else %if %upcase(&srce_dsn) eq ENTRYFILTER %then %do;
      attrib Id
               length  =$36
             Label
               length  =$128
             Entry
               length  =$36
             DbColumn
               length  =$36
             Operator
               length  =$10
             FilterValue
               length  =$255
         ;
   %end;
   %else %if %upcase(&srce_dsn) eq FIELD %then %do;
      attrib Id
               length  =$36
             Label
               length  =$36
             Entry
               length  =$36
             DbColumn
               length  =$36
             Tag
               length  =$36
             Link
               length  =$255
             Sortnum
               length  =8
             SortOrder
               length  =8
             NumRows
               length  =8
             IsDisplayOrdinal
               length  =8
             IsClass
               length  =8
             IsHidden
               length  =8
             IsPercentage
               length  =8
             IsPercentageClass
               length  =8
             Formula
               length  =$255
         ;
   %end;
   %else %if %upcase(&srce_dsn) eq GRAPH %then %do;
      attrib Id
               length  =$36
             Label
               length  =$36
             Description
               length  =$255
             Definition
               length  =$36
             GraphType
               length  =$36
             Style
               length  =$36
             Scheme
               length  =$36
             Width
               length  =8
             Height
               length  =8
         ;
   %end;
   %else %if %upcase(&srce_dsn) eq GRAPHVARIABLE %then %do;
      attrib Id
               length  =$36
             Label
               length  =$128
             Graph
               length  =$36
             Field
               length  =$36
             Axis
               length  =8
         ;
   %end;
   %else %if %upcase(&srce_dsn) eq INPUTVALUE %then %do;
      attrib Id
               length  =$36
             Datasource
               length  =$36
             Name
               length  =$128
             InputValue
               length  =$128
             SourceDb
               length  =$36
             SourceTable
               length  =$36
         ;
   %end;
   %else %if %upcase(&srce_dsn) eq PARM %then %do;
      attrib Id
               length  =$36
             Program
               length  =$36
             Name
               length  =$128
             ParmValue
               length  =$128
         ;
   %end;
   %else %if %upcase(&srce_dsn) eq PROGRAM %then %do;
      attrib Id
               length  =$36
             Label
               length  =$128
             Datasource
               length  =$36
             Code
               length  =$128
             Description
               length  =$255
         ;
   %end;
   %* end of the DOCUMENT library attributes --------------------------------- ;

   %* start of the SUMMARY library attributes -------------------------------- ;
   %***************************************************************************;
   %*   these are NOT all the fields in PATH analysis data sets, just the ones ;
   %*      that need to have common attributes                                 ;
   %***************************************************************************;
   %else %if %upcase(&srce_dsn) eq PATHS %then %do;
      attrib Count
               length=8
               format=NLNUM12.
               label ="Visits"
             Support
               length=8
               format=NLNUM6.2
               label="% Visits"
             Conf
               length=8
               format=NLNUM6.2
               label="Path Likeliness"
         ;
   %end;
   %* end of the SUMMARY library attributes ---------------------------------- ;


   %else %do;
      %put %unquote(&wab_error) %upcase(&srce_dsn) does not %QCMPRES(have
           standard) Web Analytics Metadata Variable Attributes.;
      %let &retcode=1;
   %end;

%mend wametatr;


%******************************************************************************;
%* Name:     watstdsn                                                          ;
%* Purpose:  Test whether a SAS data set has the correct variables and         ;
%*           attributes.                                                       ;
%* Usage:    %watstdsn(test_dsn=config.wasumeng                                ;
%*                    ,tmpl_dsn=wasumeng                                       ;
%*                    ,retcode=wab_rc                                          ;
%*                    )                                                        ;
%******************************************************************************;
%macro watstdsn(test_dsn=
               ,tmpl_dsn=
               ,retcode=
               );

   %local macname;
   %let   macname=&sysmacroname;

   %let &retcode=0;

   %***************************************************************************;
   %*   Check whether &TEST_DSN exists                                         ;
   %***************************************************************************;
   %if %sysfunc(exist(&test_dsn))=0 %then %do;
      %put %unquote(&wab_error) The &test_dsn data set does not exist.;
      %let &retcode=1;
      %goto ERREXIT;
   %end;

   %***************************************************************************;
   %*   Create empty data set with std set of variables (& attributes)         ;
   %***************************************************************************;
   data _wab_template;
      %wametatr(srce_dsn=&tmpl_dsn
               ,retcode=wab_rc
               );
      stop;
   run;
   %if &wab_rc=1  %then %do; %* %WAMETATR failed                               ;
      %goto ERREXIT;
   %end;

   %***************************************************************************;
   %*   Identify the LIBNAME/MEMNAME for &TEST_DSN                             ;
   %***************************************************************************;
   data _null_;
      length libname
             memname $32;
      if not index("&test_dsn",'.') then do;
         libname='WORK';
         memname=upcase("&test_dsn");
      end;
      else do;
         libname=upcase(scan("&test_dsn",1,'.'));
         memname=upcase(scan("&test_dsn",2,'.'));
      end;
      call symput('test_lib',strip(libname));
      call symput('test_mem',strip(memname));
   run;

   %***************************************************************************;
   %* retrieve info about vars in template and &TEST_DSN                       ;
   %***************************************************************************;
   proc contents data=work._wab_template noprint
              out =compare
               (keep=libname
                     memname
                     name
                     npos
                     varnum
                     type
                     length
                     label
                     format
                     informat);
   run;
   proc contents data=&test_lib..&test_mem noprint
              out =base
               (keep=libname
                     memname
                     name
                     npos
                     varnum
                     type
                     length
                     label
                     format
                     informat);
   run;

   %***************************************************************************;
   %* compare the retrieved information                                        ;
   %***************************************************************************;
   proc compare base=base       (drop=libname memname)
                compare=compare (drop=libname memname)
                out=_wab_comp_results
                  outdif
                noprint;
      by name;
   run;

   %if %get_observation_count(indsn=_wab_comp_results)=0 %then %do;
      %put %unquote(&wab_error) %upcase(&test_dsn) set does not exist.;
      %let &retcode=1;
   %end;

   data _null_;
      set _wab_comp_results end=eof;

      error_idx+(sum(abs(length),abs(npos),abs(varnum)));

      if eof then do;
         if error_idx ne 0 then
            error_idx=1;
         call symput("&retcode",strip(put(error_idx,1.)));
      end;

   run;

   %if &wab_rc=1  %then %do;
      %put %unquote(&wab_error) %upcase(&test_dsn) set does not %QCMPRES(have
           the) standard variables/attributes.;
   %end;

%ERREXIT:;


%mend watstdsn;


%******************************************************************************;
%* Copyright (C) 2000 by SAS Institute Inc., Cary, NC 27512-8000               ;
%*                                                                             ;
%* Name:     Current_OS                                                        ;
%*                                                                             ;
%* Purpose:  Determine current operating system                                ;
%*                                                                             ;
%* Output:   global macro variables:                                           ;
%*              operating_system -- valid values: MVS, OSYS or WIN             ;
%*              separator -- valid values: /, \, or (                          ;
%*                                                                             ;
%* Usage:    %Current_OS;                                                      ;
%* Notes:    Determines the current operating system based on the              ;
%*           values of the global system macro variable SYSSCP                 ;
%*                                                                             ;
%*                                                                             ;
%******************************************************************************;
%macro Current_OS;

%global operating_system
        separator
        make_dir
        remove_dir
        temp_dir
        find_options;

options nonotes;
data _null_;

    select;
       when (upcase("&sysscp") in ("SUN 64","HP IPF","AIX 64")) do;
          call symput("operating_system","OSYS") ;
          call symput("separator","/") ;
          call symput("make_dir","mkdir -p");
          call symput("remove_dir","rm -Rf");
          call symput("delete","/bin/rm");
          call symput("delete_all","/bin/rm");
          call symput("temp_dir","/tmp");
          call symput("find_options",
               "! -type d -follow -exec /bin/ls -Ls {} \;");
       end ;
       %* Includes                               ;
       %* Windows NT                   (WIN_NT)  ;
       %* Windows NT Server            (WIN_NTSV);
       %* Windows 2000 Professional    (WIN_PRO) ;
       %* Windows 2000 Server          (WIN_SRV) ;
       %* Windows 2000 Advanced Server (WIN_ASRV);
       %* Windows 2000 DC Server       (WIN_DSRV);
       %* Windows 95                   (WIN_94)  ;
       %* Windows 98                   (WIN_98)  ;
       %* Windows Millennium           (WIN_ME)  ;
       when (upcase("&sysscp") in ("WIN","DNTHOST")) do ;
          call symput("operating_system","WIN") ;
          call symput("separator",'\') ;
          call symput("make_dir","md");
          call symput("remove_dir", "rd /s /q");
          call symput("delete","erase");
          call symput("delete_all","erase /Q");

          %* Assign Temporary directory based on environment ;
          %* variable TEMP                                   ;
          filename tmp '!TEMP';
          rc=fileref("tmp");
          if (rc=0) then do;
             fname=pathname("tmp");
             if fname eq "" then do;
                fname = "C:\TEMP";
             end;
          end;
          else do;
             fname = "C:\TEMP";
          end;
          call symput("temp_dir", trim(fname));
          filename tmp clear;
       end ;
       otherwise do;
         call symput('_wab_rc_', '1');
       end;
    end ;

run;

%if &_wab_rc_ eq 1 %then %do;
   %put %unquote(&wab_error) An error has occurred. Unknown operating system.;
   %put %unquote(&wab_error) Value of sysscpl is: &sysscpl..;
   %put %unquote(&wab_error) Value of sysscp is : &sysscp..;
%end;

%Maybe_Notes;

%mend Current_OS;


%******************************************************************************;
%* Name:     Make_Directory                                                    ;
%* Purpose:  Turn off options notes based on wbdebug flag                      ;
%* Usage:    %Make_Directory                                                   ;
%******************************************************************************;
%macro make_directory(dirname= ,retcode=);

   %local macname;
   %let   macname=&sysmacroname;

   %let &retcode=0;   %* Initialize return code ;

   %if not %sysfunc(fileexist(&dirname)) %then %do ;

      %put %unquote(&wab_note) Creating directory &dirname;

      %No_Notes;

      %execcmd(&make_dir "&dirname");

      %if not %sysfunc(fileexist(&dirname)) %then %do ;
         %put %unquote(&wab_error) Cannot create directory: &dirname;
         %put %unquote(&wab_error) Check path and operating system permissions;
         %let &retcode=1;
      %end;

      %maybe_notes;

   %end;

%mend make_directory;


%******************************************************************************;
%* Name:     Remove_Directory                                                  ;
%* Purpose:  Remove a directory (and all its sub-directories)                  ;
%* Usage:    %Remove_Directory                                                 ;
%******************************************************************************;
%macro remove_directory(rempath=
                       ,retcode=);

   %local macname;
   %let   macname=&sysmacroname;

   %let &retcode=0;   %* Initialize return code ;

   %if %sysfunc(fileexist(&rempath)) %then %do ;
      %put %unquote(&wab_note) Removing directory &rempath.;
      data _null_;
         length cmd_str $200;
         select;
            when (upcase("&sysscp") in ("SUN 64","HP IPF","AIX 64")) do;
               cmd_str='rm '
                  || "&rempath"
                  || ' -Rf';
            end ;
            when (upcase("&sysscp") in ("WIN","DNTHOST")) do ;
               cmd_str='rd '
                  || "&rempath"
                  || ' /s /q';
            end;
         end;
         call system(cmd_str);
      run;

      %if %sysfunc(fileexist(&rempath)) %then %do ;
         %put %unquote(&wab_error) Cannot remove directory: &rempath.;
         %put %unquote(&wab_error) Check path and operating system permissions;
         %let &retcode=1;
      %end;

   %end;

%mend remove_directory;


%******************************************************************************;
%*                                                                             ;
%* Copyright (c) 2004 by SAS Institute Inc., Cary, NC 27512-8000               ;
%*                                                                             ;
%* Type: Macro                                                                 ;
%*                                                                             ;
%* Program: get_observation_count.sas (Web Analytics)                          ;
%*                                                                             ;
%* Purpose: To get the number of observations in a data set.                   ;
%*                                                                             ;
%* Dependencies:                                                               ;
%*    The following macro variables need to be defined and assigned values     ;
%*       in the invoking environment:                                          ;
%*    &WAB_NOTE:  the text that identifies WA macro notification messages.     ;
%*    &WAB_ERROR: the text that identifies WA macro errors.                    ;
%*                                                                             ;
%* Parameters:                                                                 ;
%*    INDSN:      (REQUIRED) The input data set whose observations are to be   ;
%*                retrieved.                                                   ;
%*    RETCODE:    mechanism for passing Return Codes (obsolete)                ;
%*                                                                             ;
%* Output:                                                                     ;
%*    NOBS:       local macro variable -- the number of obs in INDSN.          ;
%*                                                                             ;
%******************************************************************************;
%macro get_observation_count(indsn=,retcode=);

%local macname;
%let   macname=&sysmacroname;

%local nobs;
%let nobs=.;

%******************************************************************************;
%* open the data set of interest.                                              ;
%******************************************************************************;
%let indsnid=%sysfunc(open(&indsn));

%******************************************************************************;
%* if the open is successful then use ATTRN to get the number of observations. ;
%*    be sure to close INDSN.                                                  ;
%******************************************************************************;
%if &indsnid %then %do;
   %put %unquote(&wab_note) Successfully opened &indsn;
   %let nobs=%sysfunc(attrn(&indsnid,nlobs));
   %let rc=%sysfunc(close(&indsnid));
%end;
%else %do;
   %put %unquote(&wab_error) unable to open &indsn - %sysfunc(sysmsg());
%end;

%******************************************************************************;
%* because the following line is the only text that is not part of a macro     ;
%*    statement, NOBS will be effectively returned to the invoking environment ;
%*    as though the GET_OBSERVATION_COUNT macro were a function.  however, any ;
%*    other non-macro statement within the GET_OBSERVATION_COUNT macro         ;
%*    (including an extraneous semicolon!) will also be part of this exchange. ;
%*    be especially careful when updating this macro to add only macro         ;
%*    statements or you will distrupt its current functionality.               ;
%******************************************************************************;
&nobs

%mend get_observation_count;


/*----------------------Web Analytics-----------------------------------------*/
/* LOCKTEST                                                                   */
/* Test whether a given data set is locked.                                   */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Supported By: Frank Roediger, FRROED                                       */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* History:                                                                   */
/* Date          Description                    Username         Change Code  */
/* 20040420      Program Created                 frroed              NA       */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Dependencies:                                                              */
/*    None                                                                    */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Parameters:                                                                */
/*    DSET:    the name of the data set to be tested                          */
/*    LOCK_SW: the mechanism for indicating whether the specified data set is */
/*             locked.                                                        */
/*                0 - data set exists and is not locked                       */
/*                1 - data set exists but is locked                           */
/*                . - data  set does not exist                                */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Input:                                                                     */
/*    (dset=                                                                  */
/*    );                                                                      */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Output:                                                                    */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Copyright (c) 2004 SAS Institute Inc. All Rights Reserved                  */
/*----------------------------------------------------------------------------*/
%macro locktest(dset=);
   %global lock_sw;
   %let lock_sw=0;

   %***************************************************************************;
   %*   make sure that the DSET parameter has both libname and memname         ;
   %*      (assume libname=WORK if there is no embedded period in &DSET).      ;
   %***************************************************************************;
   %if not %index(&dset,.) %then %do;
      %let dset=work.&dset;
   %end;

   %let open_rc=%sysfunc(open(&dset));
   %if &open_rc %then %do;
      %let close_rc=%sysfunc(close(&open_rc));
      lock &dset query;
      %if &syslckrc eq -630099 %then %do;
         %* &DSET exists and is not locked;
         %let lock_sw=0;
      %end;
      %else %do;
         %* &DSET exists and is locked;
         %let lock_sw=1;
      %end;
   %end;
   %else %do;
      %* &DSET does not exist;
      %let lock_sw=.;
   %end;

%mend locktest;


/*----------------------Web Analytics-----------------------------------------*/
/* WAMETDEF                                                                   */
/* Define and create the Web Analytics metadata data sets                     */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Supported By: Frank Roediger, FRROED                                       */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* History:                                                                   */
/* Date          Description                    Username         Change Code  */
/* 20040831      Program Created                 frroed              NA       */
/* 20041007      Added DEF_LIB parameter         frroed              C001     */
/* 20041110      Added label to WAAUTOSG,        cabahl           S0275612    */
/*               WADSHBRD and WASCRCRD renamed scorecard, dashboard and       */
/*               segmentation_name to name                                    */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Dependencies:                                                              */
/*    %WAMETATR                                                               */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Parameters:                                                                */
/*    DEF_DSN: the name of the data set to be defined/created                 */
/*    DEF_LIB: the LIBNAME for &DEF_DSN                                       */
/*    RETCODE: the mechanism for indicating whether the macro ran             */
/*             successfully                                                   */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Input:                                                                     */
/*    (def_dsn=                                                               */
/*    ,def_lib=config                                                         */
/*    ,retcode=                                                               */
/*    );                                                                      */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Output:                                                                    */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Copyright (c) 2004 SAS Institute Inc. All Rights Reserved                  */
/*----------------------------------------------------------------------------*/
%macro wametdef(def_dsn=
               ,def_lib=config
               ,retcode=
               );


   %local macname;
   %let   macname=&sysmacroname;

   %let &retcode=0;


%******************************************************************************;
%*   MODULE: Create WASUMENG                                                   ;
%*   SUPPORT:frroed                                                            ;
%******************************************************************************;
%if %upcase(&def_dsn) eq WASUMENG %then %do;
   %***************************************************************************;
   %*   NOTE: this block was removed when WAADMSUM replaced WASUMENG           ;
   %***************************************************************************;
%end;


%******************************************************************************;
%*  MODULE: Create WASCRINP                                                    ;
%*  SUPPORT:sassyw                                                             ;
%******************************************************************************;
%else %if %upcase(&def_dsn) eq WASCRINP %then %do;
   data &def_lib..wascrinp;

      %wametatr(srce_dsn=wascrinp
               ,retcode=wab_rc
               );
      %if &wab_rc ne 0 %then %do;
         %put %unquote(&wab_error) Attributes cannot be assigned %QCMPRES(
              for) the variables in the &def_dsn data set.;
         stop;
         run;
         proc datasets library=&def_lib nolist;
            delete &def_dsn;
         quit;
         %goto ERREXIT;
      %end;

      id=id;           %* avoid the "Variable id is uninitialized" log notice  ;

      scorecard_id ='01';
      role         ='input';
      _name_       ='err_501_count';
      _label_      ='501 Count: Not Implemented';
      business_dir ='DOWN';
      measurement_range='POSITIVE NUMBER';
      link         =' ';
      output;

      scorecard_id ='01';
      role         ='input';
      _name_       ='file_count';
      _label_      ='Hit Count';
      business_dir ='UP';
      measurement_range='POSITIVE NUMBER';
      link         =' ';
      output;

      scorecard_id ='01';
      role         ='input';
      _name_       ='page_count';
      _label_      ='Page Count';
      business_dir ='UP';
      measurement_range='POSITIVE NUMBER';
      link         =' ';
      output;

      scorecard_id ='01';
      role         ='target';
      _name_       ='session_count';
      _label_      ='Visit Count';
      business_dir ='UP';
      measurement_range='POSITIVE NUMBER';
      link         =' ';
      output;

      scorecard_id ='01';
      role         ='input';
      _name_       ='duration';
      _label_      ='Duration in Seconds';
      business_dir ='UP';
      measurement_range='POSITIVE NUMBER';
      link         =' ';
      output;

      scorecard_id ='01';
      role         ='input';
      _name_       ='total_bytes_sent';
      _label_      ='Total Bytes Sent';
      business_dir ='UP';
      measurement_range='POSITIVE NUMBER';
      link         =' ';
      output;

   run;


   data &def_lib..wascrinp;
      set &def_lib..wascrinp;

      seq+1;
      id=strip(put(seq,6.));

      drop seq;

   run;


   proc datasets library=&def_lib nolist;
      modify wascrinp;
         index create id
             / nomiss
               unique;
   quit;
%end;


%******************************************************************************;
%*   MODULE: Create WASCRCRD                                                   ;
%*   SUPPORT:sassyw                                                            ;
%******************************************************************************;
%else %if %upcase(&def_dsn) eq WASCRCRD %then %do;
   data &def_lib..wascrcrd;

      %wametatr(srce_dsn=wascrcrd
               ,retcode=wab_rc
               );
      %if &wab_rc ne 0 %then %do;
         %put %unquote(&wab_error) Attributes cannot be assigned %QCMPRES(
              for) the variables in the &def_dsn data set.;
         stop;
         run;
         proc datasets library=&def_lib nolist;
            delete &def_dsn;
         quit;
         %goto ERREXIT;
      %end;

      id    ='01';
      name  ='Site Metrics';
      indsn ='summary.daily_total_day';
      label ='Site Metrics';
      output;

   run;


   proc datasets library=&def_lib nolist;
      modify wascrcrd;
         index create id
             / nomiss
               unique;
   quit;

%end;


%******************************************************************************;
%*   MODULE: Create WADSHINP                                                   ;
%*   SUPPORT:sassyw                                                            ;
%******************************************************************************;
%else %if %upcase(&def_dsn) eq WADSHINP %then %do;
   data &def_lib..wadshinp;

      %wametatr(srce_dsn=wadshinp
               ,retcode=wab_rc
               );
      %if &wab_rc ne 0 %then %do;
         %put %unquote(&wab_error) Attributes cannot be assigned %QCMPRES(
              for) the variables in the &def_dsn data set.;
         stop;
         run;
         proc datasets library=&def_lib nolist;
            delete &def_dsn;
         quit;
         %goto ERREXIT;
      %end;

      id=id;           %* avoid the "Variable id is uninitialized" log notice  ;

      dashboard_id ='01';
      time_span    ='day';
      role         ='metric';
      _name_       ='err_302_count';
      _name_category_='Site Health';
      _label_      ='302 Count: Found';
      business_dir ='DOWN';
      link         =' ';
      output;

      dashboard_id ='01';
      time_span    ='day';
      role         ='metric';
      _name_       ='err_304_count';
      _name_category_='Site Health';
      _label_      ='304 Count: Not Modified';
      business_dir ='DOWN';
      link         =' ';
      output;

      dashboard_id ='01';
      time_span    ='day';
      role         ='metric';
      _name_       ='err_400_count';
      _name_category_='Site Health';
      _label_      ='400 Count: Bad Request';
      business_dir ='DOWN';
      link         =' ';
      output;

      dashboard_id ='01';
      time_span    ='day';
      role         ='metric';
      _name_       ='err_401_count';
      _name_category_='Site Health';
      _label_      ='401 Count: Unauthorized';
      business_dir ='DOWN';
      link         =' ';
      output;

      dashboard_id ='01';
      time_span    ='day';
      role         ='metric';
      _name_       ='err_403_count';
      _name_category_='Site Health';
      _label_      ='403 Count: Forbidden';
      business_dir ='DOWN';
      link         =' ';
      output;

      dashboard_id ='01';
      time_span    ='day';
      role         ='metric';
      _name_       ='err_404_count';
      _name_category_='Site Health';
      _label_      ='404 Count: Not Found';
      business_dir ='DOWN';
      link         =' ';
      output;

      dashboard_id ='01';
      time_span    ='day';
      role         ='metric';
      _name_       ='err_405_count';
      _name_category_='Site Health';
      _label_      ='405 Count: Method Not Allowed';
      business_dir ='DOWN';
      link         =' ';
      output;

      dashboard_id ='01';
      time_span    ='day';
      role         ='metric';
      _name_       ='err_408_count';
      _name_category_='Site Health';
      _label_      ='408 Count: Request Time-out';
      business_dir ='DOWN';
      link         =' ';
      output;

      dashboard_id ='01';
      time_span    ='day';
      role         ='metric';
      _name_       ='err_500_count';
      _name_category_='Site Health';
      _label_      ='500 Count: Internal Server Error';
      business_dir ='DOWN';
      link         =' ';
      output;

      dashboard_id ='01';
      time_span    ='day';
      role         ='metric';
      _name_       ='err_501_count';
      _name_category_='Site Health';
      _label_      ='501 Count: Not Implemented';
      business_dir ='DOWN';
      link         =' ';
      output;

      dashboard_id ='01';
      time_span    ='day';
      role         ='metric';
      _name_       ='file_count';
      _name_category_='Traffic';
      _label_      ='Hit Count';
      business_dir ='UP';
      link         =' ';
      output;

      dashboard_id ='01';
      time_span    ='day';
      role         ='metric';
      _name_       ='One_Hit_session_count';
      _name_category_='Traffic';
      _label_      ='One Hit Visits';
      link         =' ';
      business_dir ='DOWN';
      output;

      dashboard_id ='01';
      time_span    ='day';
      role         ='metric';
      _name_       ='page_count';
      _name_category_='Traffic';
      _label_      ='Page Count';
      business_dir ='UP';
      link         =' ';
      output;

      dashboard_id ='01';
      time_span    ='day';
      role         ='metric';
      _name_       ='session_count';
      _name_category_='Traffic';
      _label_      ='Visit Count';
      business_dir ='UP';
      link         =' ';
      output;

      dashboard_id ='01';
      time_span    ='day';
      role         ='metric';
      _name_       ='duration';
      _name_category_='Traffic';
      _label_      ='Duration in Seconds';
      business_dir ='UP';
      link         =' ';
      output;

      dashboard_id ='01';
      time_span    ='day';
      role         ='metric';
      _name_       ='page_view_2to4_ind';
      _name_category_='Traffic';
      _label_      ='2-4 Page Visit Count';
      business_dir ='UP';
      link         =' ';
      output;

      dashboard_id ='01';
      time_span    ='day';
      role         ='metric';
      _name_       ='page_view_le1_ind';
      _name_category_='Traffic';
      _label_      ='0-1 Page Visit Count';
      business_dir ='DOWN';
      link         =' ';
      output;

      dashboard_id ='01';
      time_span    ='day';
      role         ='metric';
      _name_       ='page_view_ge5_ind';
      _name_category_='Traffic';
      _label_      ='>= 5 Page Visit Count';
      business_dir ='UP';
      link         =' ';
      output;

      dashboard_id ='01';
      time_span    ='day';
      role         ='metric';
      _name_       ='total_bytes_sent';
      _name_category_='Traffic';
      _label_      ='Total Bytes Sent';
      business_dir ='UP';
      link         =' ';
      output;

   run;


   data &def_lib..wadshinp;
      set &def_lib..wadshinp;

      seq+1;
      id=strip(put(seq,6.));

      drop seq;

   run;


   proc datasets library=&def_lib nolist;
      modify wadshinp;
         index create id
             / nomiss
               unique;
   quit;



%end;


%******************************************************************************;
%*   MODULE: Create WADSHBRD                                                   ;
%*   SUPPORT:sassyw                                                            ;
%******************************************************************************;
%else %if %upcase(&def_dsn) eq WADSHBRD %then %do;
   data &def_lib..wadshbrd;

      %wametatr(srce_dsn=wadshbrd
               ,retcode=wab_rc
               );
      %if &wab_rc ne 0 %then %do;
         %put %unquote(&wab_error) Attributes cannot be assigned %QCMPRES(
              for) the variables in the &def_dsn data set.;
         stop;
         run;
         proc datasets library=&def_lib nolist;
            delete &def_dsn;
         quit;
         %goto ERREXIT;
      %end;


      id    ='01';
      name  ='Site Metrics';
      indsn ='summary.daily_total_day';
      label ='Site Metrics';
      output;

   run;

   proc datasets library=&def_lib nolist;
      modify wadshbrd;
         index create id
             / nomiss
               unique;
   quit;

%end;


%******************************************************************************;
%*   MODULE: Create WADTSORT                                                   ;
%*   SUPPORT:frroed                                                            ;
%*   NOTE: the libref for WADTSORT is always SUMMARY so that there will not be ;
%*         any issue about authorization/access when creating this data set    ;
%*         (depending upon the way that SWA is set up and how user permissions ;
%*         are assigned, writing WADTSORT to the CONFIG library might fail)    ;
%******************************************************************************;
%else %if %upcase(&def_dsn) eq WADTSORT %then %do;

   data summary.wadtsort;

      %wametatr(srce_dsn=wadtsort
               ,retcode=wab_rc
               );
      %if &wab_rc ne 0 %then %do;
         %put %unquote(&wab_error) Attributes cannot be assigned %QCMPRES(
              for) the variables in the &def_dsn data set.;
         stop;
         run;
         proc datasets library=&def_lib nolist;
            delete &def_dsn;
         quit;
         %goto ERREXIT;
      %end;

      do date=today() to (today()+6);
         Dow_ID=strip(put(weekday(date),2.));

         start_of_week=intnx("week&wab_week_shift",date,0);
         weekday_sow  =weekday(start_of_week);

         Dow_Sort=1 + (weekday(date)-weekday_sow);
         if dow_sort eq 0 then
            Dow_Sort=7;

         Dow_Name=strip(put(date,downame.));
         output;
      end;

      keep dow_id
           dow_name
           dow_sort;

   run;

   proc sort data=summary.wadtsort;
      by Dow_Sort;
   run;

%end;


%******************************************************************************;
%*   MODULE: Create WACONFIG                                                   ;
%*   SUPPORT:saskxs                                                            ;
%******************************************************************************;
%else %if %upcase(&def_dsn) eq WACONFIG %then %do;

   data &def_lib..waconfig;

      %wametatr(srce_dsn=waconfig
               ,retcode=wab_rc
               );
      %if &wab_rc ne 0 %then %do;
         %put %unquote(&wab_error) Attributes cannot be assigned %QCMPRES(
              for) the variables in the &def_dsn data set.;
         stop;
         run;
         proc datasets library=&def_lib nolist;
            delete &def_dsn;
         quit;
         %goto ERREXIT;
      %end;


      id=id;           %* avoid the "Variable id is uninitialized" log notice  ;

      Parameter_Name='wab_r1_support';
      Value='10';
      Description='The minimum count of total sessions for path for 1 day.';
      output;

      Parameter_Name='wab_r7_support';
      Value='40';
      Description='The minimum count of total sessions for path for 7 day.';
      output;

      Parameter_Name='wab_r30_support';
      Value='100';
      Description='The minimum count of total sessions for path for 30 days.';
      output;

      Parameter_Name='wab_itemset_size';
      Value='7';
      Description='The maximum number of URLs/Pages for a path.';
      output;

      Parameter_Name='wab_metadsn';
      Value='&temp_lib..wasumeng';
      Description='Summary Engine WASUMENG metadata which drives the creation '
               || 'of the proc summary code to build the descriptive '
               || 'summaries in the warehouse.';
      output;

      Parameter_Name='wab_admsum';
      Value='config.waadmsum';
      Description='Summary Engine Administration WAADMSUM metadata which '
               || 'is the source for the WASUMENG metadata.';
      output;

      Parameter_Name='wab_days_in_history';
      Value='90';
      Description= 'The number of days available in daily summaries';
      output;

      Parameter_Name='wab_weeks_in_history';
      Value='52';
      Description= 'The number of weeks available in weekly summaries';
      output;

      Parameter_Name='wab_months_in_history';
      Value='36';
      Description= 'The number of months available in monthly summaries';
      output;

      Parameter_Name='wab_quarters_in_history';
      Value='12';
      Description= 'The number of quarters available in quarterly summaries';
      output;

      Parameter_Name='wab_years_in_history';
      Value='3';
      Description= 'The number of years available in yearly summaries';
      output;

      Parameter_Name='wab_num_detail_data_sets';
      Value='7';
      Description= 'The number of separate date partitioned detail'
                 ||' data sets to keep.';
      output;

      Parameter_Name='wab_num_session_data_sets';
      Value='30';
      Description= 'The number of separate date partitioned session'
                 ||' data sets to keep.';
      output;

      Parameter_Name='wab_visitor_days_in_history';
      Value='90';
      Description= 'The number of days of history that is retained'
                 ||' in SUMMARY.VISITOR_DAY.';
      output;

      Parameter_Name='wab_pathing_days_in_history';
      Value='30';
      Description= 'The number of days of history that is retained'
                 ||' in SUMMARY.PATHING.';
      output;

      Parameter_Name='wab_autoseg_train_samp_pct';
      Value='0.6';
      Description= 'The proportion of total vistor ids'
                 ||' used to build the automatic segmentation'
                 ||' model.';
      output;

      Parameter_Name='wab_autoseg_valid_samp_pct';
      Value='0.4';
      Description= 'The proportion of total vistor ids'
                 ||' used to identify/validate the automatic segmentation'
                 ||' model.';
      output;

      Parameter_Name='wab_autoseg_num_segments';
      Value='4';
      Description= 'The number of segments generated by the automatic'
                 ||' segmentation model.';
      output;

      Parameter_Name='wab_autoseg_alpha';
      Value='0.20';
      Description= 'For automatic segmentation, alpha specifies a threshold'
                 ||' p-value for the significance level of a candidate splitting'
                 ||' rule. This only applicable for targets which have interval'
                 ||' measurement level.';
      output;

      Parameter_Name='wab_autoseg_minworth';
      Value='0.0';
      Description= 'For automatic segmentation, worth specifies a threshold'
                 ||' value for the logworth of a candidate splitting rule'
                 ||' This only applicable for targets which have nominal'
                 ||' or ordinal measurement level.';
      output;

      Parameter_Name='wab_autoseg_maxdepth';
      Value='10';
      Description= 'For automatic segmentation, depth is the'
                 ||' number of splitting rules needed to define the node.'
                 ||' The root node has depth zero. The children of the root'
                 ||' node have depth one, and so on.';
      output;

      Parameter_Name='wab_autoseg_leafsize';
      Value=' ';
      Description= 'For automatic segmentation, leaf size is the'
                 ||' smallest number of training observations a new branch may have.';
      output;

      Parameter_Name='wab_init_by_sess_vars';
      Value='status_code,requested_file';
      Description= 'Comma-delimited list of INITIALIZE_BY_SESSION variables.';
      output;

      Parameter_Name='wab_days_in_scorecard';
      Value='365';
      Description= 'The number of days available for Scorecards.';
      output;

      Parameter_Name='wab_days_in_dashboard';
      Value='365';
      Description= 'The number of days available for Dashboards.';
      output;

      Parameter_Name='wab_days_in_autoseg';
      Value='365';
      Description= 'The number of days available for Segmentations.';
      output;

      Parameter_Name='wab_first_sess_vars';
      Value=' ';
      Description= 'Comma-delimited list of SESSION vars with first.session_id'
                 ||' values.';
      output;

      Parameter_Name='wab_last_sess_vars';
      Value=' ';
      Description= 'Comma-delimited list of SESSION vars with last.session_id'
                 ||' values.';
      output;

      Parameter_Name='wab_sum_sess_vars';
      Value=' ';
      Description= 'Comma-delimited list of SESSION vars with sum of all '
                 ||' session_id values.';
      output;

      Parameter_Name='wab_interactive_support';
      Value='3';
      Description= 'Minimum number of sessions per day for a path of'
                 ||' requested files to be output in interactive pathing.';
      output;

      Parameter_Name='week_start';
      Value='Sunday';
      Description= 'The English-language designation of 1st day in week.';
      output;

   run;


   data &def_lib..waconfig;
      set &def_lib..waconfig;

      seq+1;
      id=strip(put(seq,6.));

      drop seq;

   run;


   proc datasets library=&def_lib nolist;
      modify waconfig;
         index create id
             / nomiss
               unique;
   quit;

%end;


%******************************************************************************;
%*   MODULE: Create WAAUTINP                                                   ;
%*   SUPPORT:saskxs                                                            ;
%******************************************************************************;
%else %if %upcase(&def_dsn) eq WAAUTINP %then %do;
   data &def_lib..waautinp;

      %wametatr(srce_dsn=waautinp
               ,retcode=wab_rc
               );
      %if &wab_rc ne 0 %then %do;
         %put %unquote(&wab_error) Attributes cannot be assigned %QCMPRES(
              for) the variables in the &def_dsn data set.;
         stop;
         run;
         proc datasets library=&def_lib nolist;
            delete &def_dsn;
         quit;
         %goto ERREXIT;
      %end;

      id=id;           %* avoid the "Variable id is uninitialized" log notice  ;

   /* 1st Segment definition ------------------------------------------------ */
      Segmentation_ID  ='01';
      role         ='target';
      level        ='nominal';
      _name_       ='repeat_visitor';
      _label_      ='Repeat Visitor';
      output;

      Segmentation_ID  ='01';
      role         ='input';
      level        ='interval';
      _name_       ='page_count_anl';
      _label_      ='Page Count Analytic Period';
      output;

      Segmentation_ID  ='01';
      role         ='input';
      level        ='interval';
      _name_       ='session_count_anl';
      _label_      ='Visit Count Analytic Period';
      output;

      Segmentation_ID  ='01';
      role         ='input';
      level        ='interval';
      _name_       ='duration_anl';
      _label_      ='Duration Analytic Period';
      output;

      Segmentation_ID  ='01';
      role         ='new visitor indicator';
      level        ='nominal';
      _name_       ='new_visitor';
      _label_      ='New Visitor';
      output;

      Segmentation_ID  ='01';
      role         ='most current visitor';
      level        ='nominal';
      _name_       ='rptdate_visitor';
      _label_      ='Most Current Visitor';
      output;

      Segmentation_ID  ='01';
      role         ='visitor id';
      level        ='nominal';
      _name_       ='visitor_id';
      _label_      ='Visitor Id';
      output;

   /* 2nd Segment definition ------------------------------------------------ */
      Segmentation_ID  ='02';
      role         ='target';
      level        ='nominal';
      _name_       ='repeat_visitor';
      _label_      ='Repeat Visitor';
      output;

      Segmentation_ID  ='02';
      role         ='input';
      level        ='interval';
      _name_       ='page_count_avg_anl';
      _label_      ='Page Count Analytic Period';
      output;

      Segmentation_ID  ='02';
      role         ='input';
      level        ='interval';
      _name_       ='page_duration_avg_anl';
      _label_      ='Avg Page Duration';
      output;

      Segmentation_ID  ='02';
      role         ='input';
      level        ='interval';
      _name_       ='session_duration_avg_anl';
      _label_      ='Avg Visit Duration';
      output;

      Segmentation_ID  ='02';
      role         ='new visitor indicator';
      level        ='nominal';
      _name_       ='new_visitor';
      _label_      ='New Visitor';
      output;

      Segmentation_ID  ='02';
      role         ='most current visitor';
      level        ='nominal';
      _name_       ='rptdate_visitor';
      _label_      ='Most Current Visitor';
      output;

      Segmentation_ID  ='02';
      role         ='visitor id';
      level        ='nominal';
      _name_       ='visitor_id';
      _label_      ='Visitor Id';
      output;

   run;

   data &def_lib..waautinp;
      set &def_lib..waautinp;

      seq+1;
      id=strip(put(seq,6.));

      drop seq;

   run;

   proc datasets library=&def_lib nolist;
      modify waautinp;
         index create id
             / nomiss
               unique;
   quit;



%end;


%******************************************************************************;
%*   MODULE: Create WAAUTOSG                                                   ;
%*   SUPPORT:saskxs                                                            ;
%******************************************************************************;
%else %if %upcase(&def_dsn) eq WAAUTOSG %then %do;
   data &def_lib..waautosg;

      %wametatr(srce_dsn=waautosg
               ,retcode=wab_rc
               );
      %if &wab_rc ne 0 %then %do;
         %put %unquote(&wab_error) Attributes cannot be assigned %QCMPRES(
              for) the variables in the &def_dsn data set.;
         stop;
         run;
         proc datasets library=&def_lib nolist;
            delete &def_dsn;
         quit;
         %goto ERREXIT;
      %end;

      id                    ='01';
      name                  ='Repeat Visitor - Totals';
      indsn                 ='summary.unique_visitor_segmentation';
      num_segments_to_create=4;
      data_prep_include_code='wauvisit.sas';
      label                 ='Repeat Visitor - Totals';
      output;

      id                    ='02';
      name                  ='Repeat Visitor - Averages';
      indsn                 ='summary.unique_visitor_segmentation';
      num_segments_to_create=4;
      data_prep_include_code='wauvisit.sas';
      label                 ='Repeat Visitor - Averages';
      output;

   run;

   proc datasets library=&def_lib nolist;
      modify waautosg;
         index create id
             / nomiss
               unique;
   quit;

%end;


%******************************************************************************;
%*   MODULE: Create WARESRCE                                                   ;
%*   SUPPORT:sassyw                                                            ;
%******************************************************************************;
%else %if %upcase(&def_dsn) eq WARESRCE %then %do;

   data &def_lib..waresrce;

      %wametatr(srce_dsn=waresrce
               ,retcode=wab_rc
               );
      %if &wab_rc ne 0 %then %do;
         %put %unquote(&wab_error) Attributes cannot be assigned %QCMPRES(
              for) the variables in the &def_dsn data set.;
         stop;
         run;
         proc datasets library=&def_lib nolist;
            delete &def_dsn;
         quit;
         %goto ERREXIT;
      %end;

      id         ='dashboard_up_bg';
      value      ='#33cc00';
      description='Dashboard positive performance background color';
      output;

      id         ='dashboard_down_bg';
      value      ='#ff0033';
      description='Dashboard negative performance background color';
      output;

      id         ='dashboard_steady_bg';
      value      ='#0000ff';
      description='Dashboard steady performance background color';
      output;

      id         ='dashboard_up_fg';
      value      ='#ffffff';
      description='Dashboard positive performance foreground color';
      output;

      id         ='dashboard_down_fg';
      value      ='#ffffff';
      description='Dashboard negative performance foreground color';
      output;

      id         ='dashboard_steady_fg';
      value      ='#ffffff';
      description='Dashboard steady performance foreground color';
      output;

      id         ='dashboard_belowbad_image';
      value      ='../images/BelowBad.gif';
      description='Dashboard trend down - desired trend up image icon';
      output;

      id         ='dashboard_belowgood_image';
      value      ='../images/BelowGood.gif';
      description='Dashboard trend down - desired trend down image icon';
      output;

      id         ='dashboard_ok_image';
      value      ='../images/WithinLimits.gif';
      description='Dashboard steady performance image icon';
      output;

      id         ='dashboard_abovegood_image';
      value      ='../images/AboveGood.gif';
      description='Dashboard trend up - desired trend up image icon';
      output;

      id         ='dashboard_abovebad_image';
      value      ='../images/AboveBad.gif';
      description='Dashboard trend up - desired trend down image icon';
      output;

      id         ='scorecard_belowbad_image';
      value      ='../images/BelowBad.gif';
      description='Scorecard Down/Negative performance image icon';
      output;

      id         ='scorecard_belowgood_image';
      value      ='../images/BelowGood.gif';
      description='Scorecard Down/Positive performance image icon';
      output;

      id         ='scorecard_ok_image';
      value      ='../images/WithinLimits.gif';
      description='Scorecard steady performance image icon';
      output;

      id         ='scorecard_abovegood_image';
      value      ='../images/AboveGood.gif';
      description='Scorecard Up/Positive performance image icon';
      output;

      id         ='scorecard_abovebad_image';
      value      ='../images/AboveBad.gif';
      description='Scorecard Up/Negative performance image icon';
      output;

      id         ='row_threshold';
      value      ='5000';
      description='# of Obs at which user is prompted to continue a report';
      output;

      id         ='funnel_sess_cutoff';
      value      ='3';
      Description= 'The value of SESSION_HIT_COUNT for PAGE_FREQUENCIES obs'
                 ||' that qualifies a URL for the interactive funnel pull-down.';
      output;

      id         ='label_wrap';
      value      ='40';
      Description= 'The maximum label length to put into a table cell before'
                 ||' wrapping.  If set to 0, no wrapping is done.';
      output;

      id         ='node_bg';
      value      ='#c0c0c0';
      description='The color to use for the node background in diagrams';
      output;

   run;


   proc sort data=&def_lib..waresrce;
      by id;
   run;


   proc datasets library=&def_lib nolist;
      modify waresrce;
         index create id
             / nomiss
               unique;
   quit;

%end;


%******************************************************************************;
%*   MODULE: Create WATBLDSC                                                   ;
%*   SUPPORT:frroed                                                            ;
%******************************************************************************;
%else %if %upcase(&def_dsn) eq WATBLDSC %then %do;

   data &def_lib..watbldsc;

      %wametatr(srce_dsn=watbldsc
               ,retcode=wab_rc
               );
      %if &wab_rc ne 0 %then %do;
         %put %unquote(&wab_error) Attributes cannot be assigned %QCMPRES(
              for) the variables in the &def_dsn data set.;
         stop;
         run;
         proc datasets library=&def_lib nolist;
            delete &def_dsn;
         quit;
         %goto ERREXIT;
      %end;

      id         ='waconfig';
      value      ='Web Analytics Configuration';
      description='Data that define and control Web Analytics settings and thresholds';
      output;

      id         ='waresrce';
      value      ='User Interface Settings';
      description='Data that define and control the UI settings';
      output;

   run;


   proc sort data=&def_lib..watbldsc;
      by id;
   run;


   proc datasets library=&def_lib nolist;
      modify watbldsc;
         index create id
             / nomiss
               unique;
   quit;

%end;


%******************************************************************************;
%*   MODULE: Create WAADMSUM                                                   ;
%*   SUPPORT:frroed                                                            ;
%******************************************************************************;
%else %if %upcase(&def_dsn) eq WAADMSUM %then %do;

   data waadmsum;

      %wametatr(srce_dsn=waadmsum
               ,retcode=wab_rc
               );
      %if &wab_rc ne 0 %then %do;
         %put %unquote(&wab_error) Attributes cannot be assigned %QCMPRES(
              for) the variables in the &def_dsn data set.;
         stop;
         run;
         proc datasets library=&def_lib nolist;
            delete &def_dsn;
         quit;
         %goto ERREXIT;
      %end;

      id=id;           %* avoid the "Variable id is uninitialized" log notice  ;

      label                    ='Browser';
      source_table             ='session';
      proc_summary_options     ='sum missing chartype';
      input_where              ='';
      summary_level_list       ='day,week,month,qtr,year';
      class_var_list           ='date,browser';
      id_var_list              ='';
      id_var_summary_level_list='';
      analysis_var_list        ='page_count,session_count';
      output_dsn_options_0     ='';
      output_dsn_options_1     ='';
      output_dsn_where_stmt    ='';
      output_options           ='sum=';
      results_table            ='browser';
      output;

      label                    ='Browser Version';
      source_table             ='session';
      proc_summary_options     ='sum missing chartype';
      input_where              ='';
      summary_level_list       ='day,week,month,qtr,year';
      class_var_list           ='date,browser,browser_version';
      id_var_list              ='';
      id_var_summary_level_list='';
      analysis_var_list        ='page_count,session_count';
      output_dsn_options_0     ='';
      output_dsn_options_1     ='';
      output_dsn_where_stmt    ='';
      output_options           ='sum=';
      results_table            ='browser_version';
      output;

      label                    ='Daily Total';
      source_table             ='session';
      proc_summary_options     ='sum missing chartype';
      input_where              ='';
      summary_level_list       ='day,week,month,qtr,year';
      class_var_list           ='date';
      id_var_list              ='dow_id';
      id_var_summary_level_list='day';
      analysis_var_list        ='err_302_count,err_304_count,err_400_count,'
                             || 'err_401_count,err_403_count,err_404_count,'
                             || 'err_405_count,err_408_count,err_500_count,'
                             || 'err_501_count,duration,file_count,page_count,'
                             || 'one_hit_session_count,total_bytes_sent,'
                             || 'session_count,page_view_le1_ind,'
                             || 'page_view_2to4_ind,page_view_ge5_ind';
      output_dsn_options_0     ='';
      output_dsn_options_1     ='';
      output_dsn_where_stmt    ='';
      output_options           ='sum=';
      results_table            ='daily_total';
      output;

      label                    ='Hourly Metrics';
      source_table             ='detail';
      proc_summary_options     ='sum missing chartype';
      input_where              ='';
      summary_level_list       ='day,week,month,qtr,year';
      class_var_list           ='date,hour';
      id_var_list              ='';
      id_var_summary_level_list='';
      analysis_var_list        ='entry_point,file_count,page_count,bytes_sent';
      output_dsn_options_0     ='';
      output_dsn_options_1     ='rename=(entry_point=session_count)';
      output_dsn_where_stmt    ='';
      output_options           ='sum=';
      results_table            ='hourly_metrics';
      output;

      label                    ='Hourly Status';
      source_table             ='detail';
      proc_summary_options     ='sum missing chartype';
      input_where              ='';
      summary_level_list       ='day,week,month,qtr,year';
      class_var_list           ='date,hour,status_code';
      id_var_list              ='';
      id_var_summary_level_list='';
      analysis_var_list        ='status_code_by_hour,file_count,page_count';
      output_dsn_options_0     ='';
      output_dsn_options_1     ='rename=(status_code_by_hour=session_count)';
      output_dsn_where_stmt    ='';
      output_options           ='sum=';
      results_table            ='hourly_status';
      output;

      label                    ='Page';
      source_table             ='detail';
      proc_summary_options     ='sum missing chartype';
      input_where              ='';
      summary_level_list       ='day,week,month,qtr,year';
      class_var_list           ='date,requested_file';
      id_var_list              ='';
      id_var_summary_level_list='';
      analysis_var_list        ='init_by_sess_b,file_count,page_count,'
                             || 'client_error_count,server_error_count,'
                             || 'entry_page_count,exit_page_count,'
                             || 'err_302_count,err_304_count,err_400_count,'
                             || 'err_401_count,err_403_count,err_404_count,'
                             || 'err_405_count,err_408_count,err_500_count,'
                             || 'err_501_count ';
      output_dsn_options_0     ='';
      output_dsn_options_1     ='rename=(init_by_sess_b=session_count)';
      output_dsn_where_stmt    ='';
      output_options           ='sum=';
      results_table            ='page';
      output;

      label                    ='Page Status';
      source_table             ='detail';
      proc_summary_options     ='sum missing chartype';
      input_where              ='';
      summary_level_list       ='day,week,month,qtr,year';
      class_var_list           ='date,status_code,requested_file';
      id_var_list              ='';
      id_var_summary_level_list='';
      analysis_var_list        ='reqfile_statcde,file_count,page_count';
      output_dsn_options_0     ='';
      output_dsn_options_1     ='rename=(reqfile_statcde=session_count)';
      output_dsn_where_stmt    ='';
      output_options           ='sum=';
      results_table            ='page_status';
      output;

      label                    ='Platform';
      source_table             ='session';
      proc_summary_options     ='sum missing chartype';
      input_where              ='';
      summary_level_list       ='day,week,month,qtr,year';
      class_var_list           ='date,platform';
      id_var_list              ='';
      id_var_summary_level_list='';
      analysis_var_list        ='page_count,session_count';
      output_dsn_options_0     ='';
      output_dsn_options_1     ='';
      output_dsn_where_stmt    ='';
      output_options           ='sum=';
      results_table            ='platform';
      output;

      label                    ='Referrer Domain';
      source_table             ='session';
      proc_summary_options     ='sum missing chartype';
      input_where              ='';
      summary_level_list       ='day,week,month,qtr,year';
      class_var_list           ='date,referrer_domain';
      id_var_list              ='';
      id_var_summary_level_list='';
      analysis_var_list        ='page_count,session_count';
      output_dsn_options_0     ='';
      output_dsn_options_1     ='';
      output_dsn_where_stmt    ='where=(referrer_domain ne " ")';
      output_options           ='sum=';
      results_table            ='referrer_domain';
      output;

      label                    ='Referrer Entry Point';
      source_table             ='session';
      proc_summary_options     ='sum missing chartype';
      input_where              ='';
      summary_level_list       ='day,week,month,qtr,year';
      class_var_list           ='date,referrer_domain,first_requested_file';
      id_var_list              ='';
      id_var_summary_level_list='';
      analysis_var_list        ='session_count';
      output_dsn_options_0     ='';
      output_dsn_options_1     ='';
      output_dsn_where_stmt    ='where=(referrer_domain ne " ")';
      output_options           ='sum=';
      results_table            ='referrer_entry_point';
      output;

      label                    ='Referrer Page Status';
      source_table             ='detail';
      proc_summary_options     ='sum missing chartype';
      input_where              ='';
      summary_level_list       ='day,week,month,qtr,year';
      class_var_list           ='date,status_code,referrer,requested_file';
      id_var_list              ='';
      id_var_summary_level_list='';
      analysis_var_list        ='reqfile_statcde,file_count,page_count';
      output_dsn_options_0     ='';
      output_dsn_options_1     ='rename=(reqfile_statcde=session_count)';
      output_dsn_where_stmt    ='';
      output_options           ='sum=';
      results_table            ='referrer_page_status';
      output;

      label                    ='Referrer Search Term';
      source_table             ='session';
      proc_summary_options     ='sum chartype';
      input_where              ='';
      summary_level_list       ='day,week,month,qtr,year';
      class_var_list           ='date,referrer_domain,search_term';
      id_var_list              ='';
      id_var_summary_level_list='';
      analysis_var_list        ='session_count';
      output_dsn_options_0     ='';
      output_dsn_options_1     ='';
      output_dsn_where_stmt    ='';
      output_options           ='sum=';
      results_table            ='referrer_search_term';
      output;

      label                    ='Search Term';
      source_table             ='session';
      proc_summary_options     ='sum chartype';
      input_where              ='';
      summary_level_list       ='day,week,month,qtr,year';
      class_var_list           ='date,search_term';
      id_var_list              ='';
      id_var_summary_level_list='';
      analysis_var_list        ='session_count';
      output_dsn_options_0     ='';
      output_dsn_options_1     ='';
      output_dsn_where_stmt    ='';
      output_options           ='sum=';
      results_table            ='search_term';
      output;

      label                    ='Status';
      source_table             ='detail';
      proc_summary_options     ='sum missing chartype';
      input_where              ='';
      summary_level_list       ='day,week,month,qtr,year';
      class_var_list           ='date,status_code';
      id_var_list              ='';
      id_var_summary_level_list='';
      analysis_var_list        ='init_by_sess_a,file_count,page_count';
      output_dsn_options_0     ='';
      output_dsn_options_1     ='rename=(init_by_sess_a=session_count)';
      output_dsn_where_stmt    ='';
      output_options           ='sum=';
      results_table            ='status';
      output;

      label                    ='Visitor';
      source_table             ='session';
      proc_summary_options     ='missing chartype';
      input_where              ='';
      summary_level_list       ='day,week,month,qtr,year';
      class_var_list           ='visitor_id,date';
      id_var_list              ='';
      id_var_summary_level_list='';
      analysis_var_list        ='beg_datetime,end_datetime,page_count,'
                             || 'session_count,duration,page_view_le1_ind,'
                             || 'page_view_2to4_ind,page_view_ge5_ind';
      output_dsn_options_0     ='keep=visitor_id'
                             || ' date min_beg max_end page_count'
                             || ' session_count duration page_view_le1_ind'
                             || ' page_view_2to4_ind page_view_ge5_ind _type_'
                             || ' drop=beg_datetime end_datetime'
                             || ' rename=(min_beg=beg_datetime max_end='
                             || 'end_datetime)';
      output_dsn_options_1     ='keep=visitor_id date min_beg max_end page_count'
                             || ' session_count duration page_view_le1_ind'
                             || ' page_view_2to4_ind page_view_ge5_ind _type_'
                             || ' drop=beg_datetime end_datetime'
                             || ' rename=(min_beg=beg_datetime'
                             || ' max_end=end_datetime)';
      output_dsn_where_stmt    ='';
      output_options           ='sum= min(beg_datetime)=min_beg max(end_datetime)'
                             || '=max_end';
      results_table            ='visitor';
      output;

   run;


   data &def_lib..waadmsum;
      set waadmsum;

      seq+1;
      id=strip(put(seq,6.));

      drop seq;

   run;


   proc sort data=&def_lib..waadmsum;
      by id;
   run;


   proc datasets library=&def_lib nolist;
      modify waadmsum;
         index create id
             / nomiss
               unique;
   quit;
%end;


%******************************************************************************;
%*   MODULE: Create WAENGVAR                                                   ;
%*   SUPPORT:frroed                                                            ;
%******************************************************************************;
%else %if %upcase(&def_dsn) eq WAENGVAR %then %do;

   proc sql;
      create table mdat_det as
         select libname,
                memname,
                name,
                label
            from dictionary.columns
            where libname eq 'WADETAIL' and
                  index(memname,'DETAIL_');
   quit;
   proc sort data=mdat_det
             out =uniq_det
             nodupkeys;
      by name;
   run;


   proc sql;
      create table mdat_ses as
         select libname,
                memname,
                name,
                label
            from dictionary.columns
            where libname eq 'DATED' and
                  index(memname,'SESSION_');
   quit;
   proc sort data=mdat_ses
             out =uniq_ses
             nodupkeys;
      by name;
   run;


   data waengvar;

      %wametatr(srce_dsn=waengvar
               ,retcode=wab_rc
               );
      %if &wab_rc ne 0 %then %do;
         %put %unquote(&wab_error) Attributes cannot be assigned %QCMPRES(
              for) the variables in the &def_dsn data set.;
         stop;
         run;
         proc datasets library=&def_lib nolist;
            delete &def_dsn;
         quit;
         %goto ERREXIT;
      %end;

      set uniq_ses (in=in_a)
          uniq_det (in=in_b);

      id=id;

      if in_a then do;
         libname='DATED';
         memname='SESSION_yyyymmdd';
      end;
      else do;
         libname='DETAIL';
         memname='DETAIL_yyyymmdd';
      end;

   run;


   data &def_lib..waengvar;
      set waengvar;

      seq+1;
      id=put(seq,6.);

      drop seq;

   run;


   proc sort data=&def_lib..waengvar;
      by id;
   run;


   proc datasets library=&def_lib nolist;
      modify waengvar;
         index create id
             / nomiss
               unique;
   quit;

%end;
%******************************************************************************;
%*   MODULE: Create WAFUNNEL                                                   ;
%*   SUPPORT:frroed                                                            ;
%******************************************************************************;
%else %if %upcase(&def_dsn) eq WAFUNNEL %then %do;

   data wafunnel;

      %wametatr(srce_dsn=wafunnel
               ,retcode=wab_rc
               );
      %if &wab_rc ne 0 %then %do;
         %put %unquote(&wab_error) Attributes cannot be assigned %QCMPRES(
              for) the variables in the &def_dsn data set.;
         stop;
         run;
         proc datasets library=&def_lib nolist;
            delete &def_dsn;
         quit;
         %goto ERREXIT;
      %end;

      stop;

   run;


   data &def_lib..wafunnel;
      set wafunnel;

      seq+1;
      id=put(seq,6.);

      drop seq;

   run;


   proc sort data=&def_lib..wafunnel;
      by id;
   run;


   proc datasets library=&def_lib nolist;
      modify wafunnel;
         index create id
             / nomiss
               unique;
   quit;

%end;


%******************************************************************************;
%*   MODULE: Create WAPATH                                                     ;
%*   SUPPORT:frroed                                                            ;
%******************************************************************************;
%else %if %upcase(&def_dsn) eq WAPATH %then %do;

   data wapath;

      %wametatr(srce_dsn=wapath
               ,retcode=wab_rc
               );
      %if &wab_rc ne 0 %then %do;
         %put %unquote(&wab_error) Attributes cannot be assigned %QCMPRES(
              for) the variables in the &def_dsn data set.;
         stop;
         run;
         proc datasets library=&def_lib nolist;
            delete &def_dsn;
         quit;
         %goto ERREXIT;
      %end;
      stop;

   run;


   data &def_lib..wapath;
      set wapath;

      seq+1;
      id=put(seq,6.);

      drop seq;

   run;


   proc sort data=&def_lib..wapath;
      by id;
   run;


   proc datasets library=&def_lib nolist;
      modify wapath;
         index create id
             / nomiss
               unique;
   quit;

%end;


%******************************************************************************;
%*   MODULE: Create WAVISMET                                                   ;
%*   SUPPORT:frroed                                                            ;
%*   NOTE: WAVISMET, unlike the other data sets that can be created by the     ;
%*      WAMETDEF macro, is only created in WORK.  It is not created in CONFIG  ;
%*      and it cannot be customized via the Administrator UI.                  ;
%******************************************************************************;
%else %if %upcase(&def_dsn) eq WAVISMET %then %do;

   data wavismet;

      %wametatr(srce_dsn=wasumeng
               ,retcode=wab_rc
               );
      %if &wab_rc ne 0 %then %do;
         %put %unquote(&wab_error) Attributes cannot be assigned %QCMPRES(
              for) the variables in the &def_dsn data set.;
         stop;
         run;
         proc datasets library=work nolist;
            delete &def_dsn;
         quit;
         %goto ERREXIT;
      %end;

      id=id;              %* avoid the "Variable id is uninitialized" log notice  ;

      input_table='&temp_lib..visitor_day';
      summary_stmt_opts='sum missing chartype';
      where_stmt='  ';
      partition_by_date=0;
      id_vars='  ';
      class_vars='date';
      analysis_vars='unique_visitor,repeat_visitor';
      freq_vars='  ';
      weight_vars='  ';
      output_table='&temp_lib..daily_visitor_day';
      summary_outdsn_opts=' ';
      summary_outdsn_where=' ';
      summary_output_opts='sum= ';
      output;

      input_table='&temp_lib..visitor_week';
      summary_stmt_opts='sum missing chartype';
      where_stmt='  ';
      partition_by_date=0;
      id_vars='  ';
      class_vars='date';
      analysis_vars='unique_visitor,repeat_visitor';
      freq_vars='  ';
      weight_vars='  ';
      output_table='&temp_lib..daily_visitor_week';
      summary_outdsn_opts=' ';
      summary_outdsn_where=' ';
      summary_output_opts='sum= ';
      output;

      input_table='&temp_lib..visitor_month';
      summary_stmt_opts='sum missing chartype';
      where_stmt='  ';
      partition_by_date=0;
      id_vars='  ';
      class_vars='date';
      analysis_vars='unique_visitor,repeat_visitor';
      freq_vars='  ';
      weight_vars='  ';
      output_table='&temp_lib..daily_visitor_month';
      summary_outdsn_opts=' ';
      summary_outdsn_where=' ';
      summary_output_opts='sum= ';
      output;

      input_table='&temp_lib..visitor_qtr';
      summary_stmt_opts='sum missing chartype';
      where_stmt='  ';
      partition_by_date=0;
      id_vars='  ';
      class_vars='date';
      analysis_vars='unique_visitor,repeat_visitor';
      freq_vars='  ';
      weight_vars='  ';
      output_table='&temp_lib..daily_visitor_qtr';
      summary_outdsn_opts=' ';
      summary_outdsn_where=' ';
      summary_output_opts='sum= ';
      output;

      input_table='&temp_lib..visitor_year';
      summary_stmt_opts='sum missing chartype';
      where_stmt='  ';
      partition_by_date=0;
      id_vars='  ';
      class_vars='date';
      analysis_vars='unique_visitor,repeat_visitor';
      freq_vars='  ';
      weight_vars='  ';
      output_table='&temp_lib..daily_visitor_year';
      summary_outdsn_opts=' ';
      summary_outdsn_where=' ';
      summary_output_opts='sum= ';
      output;

   run;

%end;


%else %do;
   %put %unquote(&wab_error) %upcase(&def_dsn) is not %QCMPRES(a
        standard) Web Analytics Metadata Data Set.;
   %let &retcode=1;
   %goto ERREXIT;
%end;
%goto EXIT;

%ERREXIT:;
%put %unquote(&wab_error) WAMETDEF exiting due to error.;

%EXIT:;
%mend wametdef;


/*----------------------Web Analytics-----------------------------------------*/
/* WASASDIR                                                                   */
/* Create the standard set of members in the \sas sub-directory               */
/*    NOTE: each module is created only if it does not exist.  No extant      */
/*          modules will be replaced.                                         */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Supported By: Frank Roediger, FRROED                                       */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* History:                                                                   */
/* Date          Description                    Username         Change Code  */
/* 20041223      Program Created                 frroed              NA       */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Dependencies:                                                              */
/*    The following macro variables must be defined in the invoking           */
/*       environment:                                                         */
/*       -- &SWAMART                                                          */
/*       -- &SEPARATOR                                                        */
/*    The path identified by the SWAMART macro variable must have a \sas      */
/*       sub-directory                                                        */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Parameters:                                                                */
/*    RETCODE: the mechanism for indicating whether the macro ran             */
/*             successfully                                                   */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Input:                                                                     */
/*    (retcode=                                                               */
/*    );                                                                      */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Output:                                                                    */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Copyright (c) 2004 SAS Institute Inc. All Rights Reserved                  */
/*----------------------------------------------------------------------------*/
%macro wasasdir(retcode=
               );

   %***************************************************************************;
   %* DAILY.SAS                                                                ;
   %***************************************************************************;
   %let wasaspgm=daily.sas;
   %if not %sysfunc(fileexist(&swamart.&separator.sas&separator.&wasaspgm)) %then %do;
      data _null_;
         file "&swamart.&separator.sas&separator.&wasaspgm";
         put '/**                                                                                    '
           / ' * Use this program to process weblog data into the SAS Web Analytics warehouse.       '
           / ' *                                                                                     '
           / ' * IMPORTANT: You must supply the location of the weblogs to process.                  '
           / ' *       Either                                                                        '
           / ' *          A. Specify the location in the WEBLOGS macro variable below by replacing   '
           / ' *             <YOUR WEBLOG LOCATION> with the location of your weblogs. WEBLOGS can   '
           / ' *             be the full path either for a single weblog or for a directory that     '
           / ' *             contains multiple weblogs.                                              '
           / ' *       Or                                                                            '
           / ' *          B. Launch the e-Data ETL administrative GUI from an interactive SAS session'
           / ' *             using the "edataetl" command and specify the location of your weblog(s) '
           / ' *             in the appropriate field.                                               '
           / ' *       If you use method B. then you must remove the "weblog_path=&weblogs" parameter'
           / ' *       from the %edataetl call below.                                                '
           / ' */                                                                                    '
          // '%let wbrc=0;                                                                           '
          // '%MACRO SWA_ETL;                                                                        '
          // '*** TODO: specify the location of your weblogs;                                        '
           / '%let WEBLOGS=<YOUR WEBLOG LOCATION>;                                                   '
          // '%let webmart='
             "&swamart"
             ';'
          // '/* Extract raw weblog data into staging datasets. */                                   '
           / '%edataetl( data_store   =  &webmart'
             "&separator."
             'e-data-etl,                                                                            '
           / '           weblog_path  =  &weblogs,                                                   '
           / '           program      =  extract                                                     '
           / '         );                                                                            '
          // '/* Load web data into final detail dataset. */                                         '
           / '%edataetl( data_store   =  &webmart'
             "&separator."
             'e-data-etl,                                                                            '
           / '           program      =  load                                                        '
           / '         );                                                                            '
          // '/* Load web detail data into warehouse. */                                             '
           / '%if &wbrc = 0 %then %do;                                                               '
          // '   %waetl( swamart  =  &webmart,                                                       '
           / '           program  =  warehouse                                                       '
           / '         );                                                                            '
          // '%end;                                                                                  '
          // '%MEND SWA_ETL;                                                                         '
           / '%SWA_ETL;                                                                              '
            ;
      run;

      %if &syserr gt 4 %then %do;
         %put %unquote(&wab_warning) &wasaspgm %QCMPRES(
              for) &swamart could not be created in the SAS %QCMPRES(
              subdirectory) for &swamart.;
         %let &retcode=1;
      %end;
   %end;


   %***************************************************************************;
   %* SETUP_LIBS.SAS                                                           ;
   %***************************************************************************;
   %let wasaspgm=setup_libs.sas;
   %if not %sysfunc(fileexist(&swamart.&separator.sas&separator.&wasaspgm)) %then %do;
      data _null_;
         file "&swamart.&separator.sas&separator.&wasaspgm";
         put '/**                                                                        '
           / ' * This is a convience program that can be included in an interactive      '
           / ' * SAS session or batch sas program.  It will define all of the appropriate'
           / ' * libraries for your Web mart.                                            '
           / ' */                                                                        '
          // 'options '@
           ;
         if upcase("&sysscp") in ("WIN","DNTHOST") then
            put 'xsync noxwait ' @;
         put 'nofmterr compress=yes; '
          // '%let webmart='
             "&swamart"
             ';'
          // 'libname detail    "&webmart'
             "&separator.e-data-etl&separator."
             'detail";'
           / 'libname wadetail  "&webmart'
             "&separator.data&separator."
             'detail";'
           / 'libname dated     "&webmart'
             "&separator.data&separator."
             'dated";'
           / 'libname document  "&webmart'
             "&separator.data&separator."
             'documents";'
           / 'libname summary   "&webmart'
             "&separator.data&separator."
             'summary";'
           / 'libname config    "&webmart'
             "&separator.config"
             '";'
            ;
      run;
      %if &syserr gt 4 %then %do;
         %put %unquote(&wab_error) &wasaspgm for %QCMPRES(
              &swamart) could not be created in the SAS %QCMPRES(
              subdirectory) for &swamart.;
         %let &retcode=1;
      %end;
   %end;


   %***************************************************************************;
   %* STP_WADESIGN.SAS                                                         ;
   %***************************************************************************;
   %let wasaspgm=stp_wadesign.sas;
   %if not %sysfunc(fileexist(&swamart.&separator.sas&separator.&wasaspgm)) %then %do;
      data _null_;
         file "&swamart.&separator.sas&separator.&wasaspgm";
         put '%global output_dataset;';
         put '*ProcessBody;';
         put '%wadesign;';
      run;
      %if &syserr gt 4 %then %do;
         %put %unquote(&wab_error) &wasaspgm for %QCMPRES(
              &swamart) could not be created in the SAS %QCMPRES(
              subdirectory) for &swamart.;
         %let &retcode=1;
      %end;
   %end;


   %***************************************************************************;
   %* STP_WAFUNNEL.SAS                                                         ;
   %***************************************************************************;
   %let wasaspgm=stp_wafunnel.sas;
   %if not %sysfunc(fileexist(&swamart.&separator.sas&separator.&wasaspgm)) %then %do;
      data _null_;
         file "&swamart.&separator.sas&separator.&wasaspgm";
         put '%global output_dataset;';
         put '*ProcessBody;';
         put '%wafunnel;';
      run;
      %if &syserr gt 4 %then %do;
         %put %unquote(&wab_error) &wasaspgm for %QCMPRES(
              &swamart) could not be created in the SAS %QCMPRES(
              subdirectory) for &swamart.;
         %let &retcode=1;
      %end;
   %end;


   %***************************************************************************;
   %* STP_WAPATH.SAS                                                           ;
   %***************************************************************************;
   %let wasaspgm=stp_wapath.sas;
   %if not %sysfunc(fileexist(&swamart.&separator.sas&separator.&wasaspgm)) %then %do;
      data _null_;
         file "&swamart.&separator.sas&separator.&wasaspgm";
         put '%global output_dataset;';
         put '*ProcessBody;';
         put '%wapath;';
      run;
      %if &syserr gt 4 %then %do;
         %put %unquote(&wab_error) &wasaspgm for %QCMPRES(
              &swamart) could not be created in the SAS %QCMPRES(
              subdirectory) for &swamart.;
         %let &retcode=1;
      %end;
   %end;


   %***************************************************************************;
   %* STP_WASTATUS.SAS                                                         ;
   %***************************************************************************;
   %let wasaspgm=stp_wastatus.sas;
   %if not %sysfunc(fileexist(&swamart.&separator.sas&separator.&wasaspgm)) %then %do;
      data _null_;
         file "&swamart.&separator.sas&separator.&wasaspgm";
         put '*ProcessBody;';
         put '%let wab_rc=0;';
         put '%wastatus(swamart=&swamart';
         put '         ,waetl_sw=N';
         put '         ,stp_sw=yes';
         put '         ,retcode=wab_rc';
         put '         );';
      run;
      %if &syserr gt 4 %then %do;
         %put %unquote(&wab_error) &wasaspgm for %QCMPRES(
              &swamart) could not be created in the SAS %QCMPRES(
              subdirectory) for &swamart.;
         %let &retcode=1;
      %end;
   %end;


   %***************************************************************************;
   %* WAUVISIT.SAS                                                             ;
   %***************************************************************************;
   %let wasaspgm=wauvisit.sas;
   %if not %sysfunc(fileexist(&swamart.&separator.sas&separator.&wasaspgm)) %then %do;
      data _null_;
         file "&swamart.&separator.sas&separator.&wasaspgm";
         put '/**********************************************************************************'
           / '* This macro creates the input data set for the default segmentation analyses.'
           / '**********************************************************************************/'
           / '   %wauvisit;'
            ;
      run;
      %if &syserr gt 4 %then %do;
         %put %unquote(&wab_error) &wasaspgm for %QCMPRES(
              &swamart) could not be created in the SAS %QCMPRES(
              subdirectory) for &swamart.;
         %let &retcode=1;
      %end;
   %end;


%mend wasasdir;


/*----------------------Web Analytics-----------------------------------------*/
/* WALISTS                                                                    */
/* Define and create the Web Analytics UI Selection List data sets            */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Supported By: Frank Roediger, FRROED                                       */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* History:                                                                   */
/* Date          Description                    Username         Change Code  */
/* 20050111      Program Created                 frroed              NA       */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Dependencies:                                                              */
/*    -- none                                                                 */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Parameters:                                                                */
/*    RETCODE: the mechanism for indicating whether the macro ran             */
/*             successfully                                                   */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Input:                                                                     */
/*    (retcode=                                                               */
/*    );                                                                      */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Output:                                                                    */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Copyright (c) 2005 SAS Institute Inc. All Rights Reserved                  */
/*----------------------------------------------------------------------------*/
%macro walists(retcode=
               );

   %***************************************************************************;
   %* use the _YEAR data sets to get the most comprehensive list               ;
   %***************************************************************************;
   %* BROWSER                                                                  ;
   %***************************************************************************;
   proc summary data=summary.browser_year chartype n;
      class browser;
      output out=&temp_lib..list_browser
         (where=(_type_ eq '1'));
   run;
   proc sort data=&temp_lib..list_browser
               (keep=browser)
             out =summary.list_browser;
      by browser;
   run;
   %if &syserr gt 4 %then %do;
      %put %unquote(&wab_error) PROC SORT for the updated %QCMPRES(
            SUMMARY.LIST_BROWSER) data set failed.;
      %goto ERREXIT;
   %end;


   %***************************************************************************;
   %* PLATFORM                                                                 ;
   %***************************************************************************;
   proc summary data=summary.platform_year chartype n;
      class platform;
      output out=&temp_lib..list_platform
         (where=(_type_ eq '1'));
   run;
   proc sort data=&temp_lib..list_platform
               (keep=platform)
             out =summary.list_platform;
      by platform;
   run;
   %if &syserr gt 4 %then %do;
      %put %unquote(&wab_error) PROC SORT for the updated %QCMPRES(
            SUMMARY.LIST_PLATFORM) data set failed.;
      %goto ERREXIT;
   %end;

   %goto EXIT;


%ERREXIT:
%let &retcode=1;
%put %unquote(&wab_error) WALISTS exiting due to error.;

%EXIT:
%put %unquote(&wab_note) WALISTS completed.;

%mend walists;


/*----------------------Web Analytics-----------------------------------------*/
/* CVT_FMTS                                                                   */
/* Convert COMMAw.d and w.d formats in SUMMARY data sets to NLNUMw.d formats  */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Supported By: Frank Roediger, FRROED                                       */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* History:                                                                   */
/* Date          Description                    Username         Change Code  */
/* 20050125      Program Created                 frroed              NA       */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Dependencies:                                                              */
/*    -- none                                                                 */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Parameters:                                                                */
/*    LIBR: the library for the data set with variables whose formats are to  */
/*          be converted                                                      */
/*    DSET: the &LIBR data set with variables whose formats are to be         */
/*          converted                                                         */
/*    FMTS: the FORMAT statement that converts the old formats                */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Input:                                                                     */
/*    (libr=summary                                                           */
/*    ,dset=                                                                  */
/*    ,fmts=                                                                  */
/*    );                                                                      */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Output:                                                                    */
/*    &LIBR..&dset is updated in-place with &FMTS replacing existing formats  */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Copyright (c) 2005 SAS Institute Inc. All Rights Reserved                  */
/*----------------------------------------------------------------------------*/
%macro cvt_fmts(libr=summary
               ,dset=
               ,fmts=
               );
   proc datasets library=&libr nolist;
      modify &dset;
      &fmts;;
   quit;

%mend cvt_fmts;


%mend wautils;
